from . import harpia
from . import eth_motor_board
from . import wintopas
from . import beam_alignment
from . import timing_controller
from . import camera_app_client
from . import harpia_daq
from . import fast_daq


# -*- coding: utf-8 -*-
"""
Created on Tue May 15 13:22:37 2018

@author: butkus
"""

options_list = {
        'PR1':  'Internal WLSc generator for PHAROS',
        'PR2':  'Internal SH generator for PHAROS',
        'PR3':  'Internal WLSc generator for Ti:Sa systems',
        'PR4':  'External probe source from ORPHEUS',
        
        'DL1':  'Physik Instrumente VT-80',
        'DL2':  'Aerotech PRO115SL-300',
        
        'DLC1': '2 ns probe beam delay configuration',
        'DLC2': '4 ns probe beam delay configuration',
        'DLC3': '8 ns probe beam delay configuration',
        'DLC4': '8 ns probe beam delay configuration with polarization rotation',
        'DLC5': '12 ns probe beam delay configuration with polarization rotation',
        'DLC6': '6 ns probe beam delay configuration',
        
        'DM1':  'Multichannel Si NMOS UV/VIS/NIR detector (Hamamatsu S3901 + C7884, 200 - 1100 nm)',
        'DM2':  'Multichannel InGaAs NIR/Mid-IR detector (Hamamatsu C8062-01 + G9208-256W, 1100 - 2600 nm)',
        'DM3':  'Multichannel InGaAs NIR/Mid-IR detector (Hamamatsu G9203-256D, 900 - 1700 nm)',
        'DM4':  'Other multichannel detector',
        
        'DS1':  'Single-channel Si UV/VIS/NIR detector (Thorlabs Det10A/M, 200-1100 nm)',
        'DS2':  'Single-channel InGaAs NIR/Mid-IR detector (Thorlabs Det10C/M, 700 - 1800 nm)',
        'DS3':  'Single-channel InGaAs NIR/Mid-IR detector (Thorlabs Det10D/M, 1200 - 2600 nm)',
        'DS4':  'Other single-channel detector',
        
        'SP0':  'No spectrograph',
        'SP1':  'Andor Kymera 193i with 2 gratings',
        'SP2':  'Andor Shamrock 163i with 2 gratings',
        'SP3':  'Internal spectrograph without reference detection',
        'SP4':  'Internal spectrograph with reference detection',
        'SP5':  'Andor Kymera 328i with 2 gratings',
        
        'BC1':  'Manual Berek\'s compensators',
        'BC2':  'Motorized Berek\'s compensators',
        
        'SH1':  'Manual sample\'s holding unit',
        'SH2':  'Sample mover - motorized sample\'s holding unit',
        
        'NDF1': 'Manual neutral density filters',
        'NDF2': 'Motorized neutral density filters',
        
        'AD1':  'Measurement in reflection mode',
        'AD2':  'Glann-Taylor polarizer',
        'AD3':  'Reference photodiode',
        'AD4':  'Crystal rotator',
        'AD5':  'Beam profiler',
        'AD6':  'Motorized pump mirror adapter',
        'AD7':  'External beam stabilization',
        'AD8':  'Internal beam position tracking',
        'AD9':  'Cryostat adapter',
        'AD10': 'Sample stirrer',
        'AD11': 'Flash photolysis extension',
        'AD12': 'reserved',
        
        'TF1':  'HARPIA-TF with Universal TCSPC photomultiplier detector',
        'TF2':  'HARPIA-TF with High speed TCSPC hybrid detector',
        
        'TB0':  'No Harpia-TB',
        'TB1':  'Harpia-TB',
        
        'MM1':  'Microscopy module',
        'MM2':  'Motorized objective translation stage for microscopy module',
        'MM3':  'Bulk module'
        }

PR_flags = {
        0x01 << (11*4): 'PR1',
        0x02 << (11*4): 'PR2',
        0x04 << (11*4): 'PR3',
        0x08 << (11*4): 'PR4',        
        }

DL_configuration_mask = 0x07 << (10*4)
DL_configuration_values = {
        0 << (10*4): 'DLC1',
        1 << (10*4): 'DLC2',
        2 << (10*4): 'DLC3',
        3 << (10*4): 'DLC4',
        4 << (10*4): 'DLC5',
        5 << (10*4): 'DLC6'
        }

DL_mask = 0x08 << (10*4)
DL_values = {
        0 << (10*4+3): 'DL1',
        1 << (10*4+3): 'DL2'
        }

DM_flags = {
        0x01 << (9*4): 'DM1',
        0x02 << (9*4): 'DM2',
        0x04 << (9*4): 'DM3',
        0x08 << (9*4): 'DM4',
        }

DS_flags = {
        0x01 << (8*4): 'DS1',
        0x02 << (8*4): 'DS2',
        0x04 << (8*4): 'DS3',
        0x08 << (8*4): 'DS4',
        }

SP_mask = 0xF<< (7*4)
SP_values = {
        0<< (7*4): 'SP0',
        1<< (7*4): 'SP1',
        2<< (7*4): 'SP2',
        3<< (7*4): 'SP3',
        4<< (7*4): 'SP4',
        5<< (7*4): 'SP5'
        }

BC_mask = 0x01 << (6*4)
BC_values = {
        0 << (6*4): 'BC1',
        1 << (6*4): 'BC2'
        }

NDF_mask = 0x02 << (6*4)
NDF_values = {
        0 << (6*4 + 1):  'NDF1',
        1 << (6*4 + 1):  'NDF2'
        }

SH_flags = {
        0x04 << (6*4): 'SH1',
        0x08 << (6*4): 'SH2'
        }

AD_flags = {
        0x001 << (3*4): 'AD12',
        0x002 << (3*4): 'AD11',
        0x004 << (3*4): 'AD10',
        0x008 << (3*4): 'AD9',
        0x010 << (3*4): 'AD8',
        0x020 << (3*4): 'AD7',
        0x040 << (3*4): 'AD6',
        0x080 << (3*4): 'AD5',
        0x100 << (3*4): 'AD4',
        0x200 << (3*4): 'AD3',
        0x400 << (3*4): 'AD2',
        0x800 << (3*4): 'AD1'
        }

TF_flags = {
        0x01 << (2*4): 'TF1',
        0x02 << (2*4): 'TF2'
        }

TB_mask = 0x08 << (2*4)
TB_values = {
        #0x00: 'TB0',
        0x08 << (2*4): 'TB1'
        }

MM_flags = {
        0x001 << (0*4): 'MM1',
        0x002 << (0*4): 'MM2',
        0x004 << (0*4): 'MM3',
        }


code = {
        'Probe sources': {
                'flags': PR_flags
                },
        'Delay line': {
                'mask': DL_configuration_mask,
                'values': DL_configuration_values
                },
        'Delay line configuration': {
                'mask': DL_mask,
                'values': DL_values
                },
        'Multichannel detector': {
                'flags': DM_flags
                },
        'Single-channel detector': {
                'flags': DS_flags
                },
        'Spectrograph': {
                'mask': SP_mask,
                'values': SP_values
                },
        'Berek\'s rotator': {
                'mask': BC_mask,
                'values': BC_values
                },
        'Nautral density filters': {
                'mask': NDF_mask,
                'values': NDF_values
                },
        'Sample holder': {
                'flags': SH_flags
                },
        'Additional options': {
                'flags': AD_flags
                },
        'HARPIA-TF': {
                'flags': TF_flags
                },
        'HARPIA-TB': {
                'mask': TB_mask,
                'values': TB_values},
        'Microscopy extenstion': {
                'flags': MM_flags
                }
        }

def decode(model='PM5A333D6B0A'):
    '''Decodes Harpia model number PMxxxxxxxxxxx to human-readable object'''
    if (model[:2] != 'PM'):
        return None
        
    if (len(model) == 12):
        model = model + 'M00'
        
    if (len(model) != 15) and (model[12] != 'M'):
        return None

    model = model[:12] + model[13:]

    if (int(model[6])>5):
        return None

    if (int(model[3], 16) > 0x05) and (int(model[3], 16) < 0x08):
        return None

    if (int(model[11], 16) & 0x04 != 0):
        return None

    number = int(model[2:], 16)
    
    decode = []
    
    for entry in code:                
        if 'flags' in code[entry]:
            for flag in code[entry]['flags']:
                if flag & number != 0:
                    option = code[entry]['flags'][flag]
                    decode.append({'option': option, 'description': options_list.get(option)})
                    #print (option, options_list.get(option))
        if 'mask' in code[entry]:
            case = code[entry]['mask'] & number
            if code[entry]['values'].get(case):
                option = code[entry]['values'][case]
                if (options_list.get(option) is not None):
                    decode.append({'option': option, 'description': options_list.get(option)})
            #print (option, options_list.get(option))
            
    return decode
    
def encode(options):
    '''Encodes options, as list of option strings, to product number PMxxxxxxxxx'''
    number = 0
    
    for entry in code:                
        if 'flags' in code[entry]:
            for key, val in code[entry]['flags'].items():
                if val in options:
                    number += key                                
        if 'mask' in code[entry]:
            for key, val in code[entry]['values'].items():
                if val in options:
                    number+= key
                    
    model = 'PM' + hex(number+(1 << (12*4)))[3:].upper()
    
    if 'MM' in [option[:2] for option in options]:
        return model[:-2]+'M'+model[-2:]
    
    return model[:-2]


# -*- coding: utf-8 -*-
"""
Created on Wed Feb  5 15:47:29 2020

@author: butkus
"""

import struct

def float_to_hex(f):
    return int(struct.unpack('<I', struct.pack('<f', f))[0])

def hex_to_float(f):
    return float(struct.unpack('<f', struct.pack('<I', f))[0])

def bytes_array_to_int (data):
    result = 0
 
    for i, val in enumerate(data):
        result = result + (val << (i * 8))
                                 
    return result


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#==========================================================================
# Harpia REST API Interface
#--------------------------------------------------------------------------
# Copyright (c) 2018 Light Conversion (UAB MGF "Šviesos konversija")
# All rights reserved.
# www.lightcon.com
#==========================================================================

from ..common import HTTP_methods

class CameraApp(HTTP_methods):
    silent = True
    connected = False
            
    def __init__ (self, ip_address, port=20042, version='v1'):
        self.url = 'http://{}:{}/{}/'.format(ip_address, port, version)
        if version == '':
            self.url = 'http://{}:{}/'.format(ip_address, port)
            
        self.connected = True;
        if self.connected:
            print ('Camera App Service initialized at', self.url)
        
    #==============================================================================
    # /Basic
    #==============================================================================
        
    def get_beam_parameters(self):
        return self._get('/BeamProfiler/BeamParameters')
    
    def enable_beam_profiler(self):
        self._put('/BeamProfiler/IsEnabled', '1')

    def disable_beam_profiler(self):
        self._put('/BeamProfiler/IsEnabled', '0')
        
    def get_beam_profiler_status(self):
        return self._get('/BeamProfiler/IsEnabled')
    
    def set_beam_profiler_mode(self, mode=['ISO','GAUSS'][0]):
        self._put('/BeamProfiler/Mode', mode)
        
    def get_beam_profiler_mode(self):
        return self._get('/BeamProfiler/Mode')
        
    def get_camera_alldata(self, index = None):
        return self._get('/Camera/AllData')            
    
    def get_camera_exposure(self):
        return self._get('/Camera/Exposure')
    
    def set_camera_exposure(self, exposure_in_ms):
        self._put('/Camera/Exposure', str(exposure_in_ms))
        
    def get_camera_serial_number(self):
        return self._get('/Camera/SerialNumber')
    
    def get_camera_background(self):
        return self._get('/Camera/Background')
    
    def set_camera_averaging(self, avaraging):
        self._put('/Camera/Averaging', str(averaging))
        
    def get_camera_averaging(self):
        return self._get('/Camera/Averaging')
    
    def set_camera_gain(self, gain):
        self._put('/Camera/Gain', str(gain))
        
    def get_camera_gain(self):
        return self._get('/Camera/Gain')
    
    def get_camera_pixel_size(self):
        return self._get('/Camera/PixelSize')
    
    def get_camera_sensor_information(self):
        return self._get('/Camera/SensorInformation')

    
    
    
    


from .http_methods import HTTP_methods
from .udp_locator import UdpLocator
from .converters import hex_to_float, float_to_hex, bytes_array_to_int
from .motor_parameters_for_L6470 import motor_parameters_for_L6470
from .serial_tools import get_serial_ports
from .leprecan import LepreCanDevice, BytesArrayToFloat, BytesArrayToInt, FloatToInt
from .komodo import Komodo


# -*- coding: utf-8 -*-
"""
Created on Thu May 18 17:45:11 2017

@author: butkus
"""
import numpy
from enum import Enum, IntEnum
import time
import struct

from ..common.komodo import Komodo

from collections import deque
from threading import Lock        

class LepreCanDevice:
    baseId = 0x100
    km = Komodo
    
    RegisterResponseTimeout = 1.5 # in [seconds]
    RegisterResponseRetries = 3
    
    mutex = Lock()
    
    def __init__ (self, km, baseId):
        self.km = km
        self.baseId = baseId
    
    def GetRegister(self, register, index, flags=0x00):
        can_id = self.GenerateMessageId(self.baseId, FrameType.GetRegisterCommandFrame)
        
        data = self.GenerateDataFrame(frameType = FrameType.GetRegisterCommandFrame, 
                                  registerAddress = register,
                                  index = index or 0x00, 
                                  flags = 0x00, 
                                  data4bytes=0x00000000)
        
        self.mutex.acquire()
        self.km.send(can_id, data)
        recv = self.km.receive()
        self.mutex.release()
        
        return (recv[3], recv[4:])
    
    def SetRegister(self, register, index, data4bytes, flags=0x00):
        can_id = self.GenerateMessageId(self.baseId, FrameType.SetRegisterCommandFrame)
        
        data = self.GenerateDataFrame(frameType = FrameType.SetRegisterCommandFrame, 
                                  registerAddress = register,
                                  index = index or 0x00, 
                                  flags = 0x00, 
                                  data4bytes=data4bytes)
        
#        usb2can.PrintCanalMsg("send", send_msg)                    
        self.mutex.acquire()
        self.km.send(can_id, data)
        recv = self.km.receive()
        self.mutex.release()
                
        return (recv[3], recv[4:])
        
    def SetRegisterAsync(self, register, index, data4bytes, flags=0x00):
        can_id = self.GenerateMessageId(self.baseId, FrameType.SetRegisterCommandFrame)
        
        data = self.GenerateDataFrame(frameType = FrameType.SetRegisterCommandFrame, 
                                  registerAddress = register,
                                  index = index or 0x00, 
                                  flags = 0x00, 
                                  data4bytes=data4bytes)
        
#        usb2can.PrintCanalMsg("send", send_msg)                    
        self.mutex.acquire()
        self.km.send(can_id, data)
        self.mutex.release()
                
        return (0, 0)

    def float_to_hex(self, f):
        return int(struct.unpack('<I', struct.pack('<f', f))[0])

    
    def IterateCrc8Byte(self, seed, newByte):
        data = numpy.uint8(seed ^ newByte);
        for i in range(8):
            if ((data & 0x80) != 0):
                data <<= 1;
                data ^= 0x07;
            else:
                data <<= 1;    
        return numpy.uint8(data);
    
    def PrepareFrame(self, data):
        indexes = [0, 1, 2, 4, 5, 6, 7]
        
        crc = self.IterateCrc8Byte (0xff, data[0]);    
        
        for index in indexes[1:]:
            crc = self.IterateCrc8Byte (crc, data[index])
            
        data[3] = crc;
        return data
        
    def GetDataString (self, data):
        return ' '.join(["{0:#0{1}x}".format(cell,4)[2:] for cell in data])
    
    
    def GenerateMessageId (self, baseId, frameType):
        return baseId + frameType.value
    
    def GenerateDataFrame (self, frameType, registerAddress, index, flags=0x00, data4bytes=0x00000000):
        data = [0] * 8
                   
        
        if (frameType == FrameType.GetRegisterCommandFrame):
            data = self.PrepareFrame([
                    registerAddress & 0x00ff,
                    (registerAddress & 0xff00) >> 8,
                    index,
                    0x00, # will be replaced by crc
                    data4bytes & 0x000000ff,
                    (data4bytes & 0x0000ff00) >> 8,
                    (data4bytes & 0x00ff0000) >> 16,
                    (data4bytes & 0xff000000) >> 24
                    ]);     
        
        if (frameType == FrameType.SetRegisterCommandFrame):
            data = self.PrepareFrame([
                    registerAddress & 0x00ff,
                    (registerAddress & 0xff00) >> 8,
                    index,
                    0x00, # will be replaced by crc
                    data4bytes & 0x000000ff,
                    (data4bytes & 0x0000ff00) >> 8,
                    (data4bytes & 0x00ff0000) >> 16,
                    (data4bytes & 0xff000000) >> 24
                    ]);   
        return data

def BytesArrayToInt (data):
    result = 0
 
    for i, val in enumerate(data):
        result = result + (val << (i * 8))
                                 
    return result     

def BytesArrayToFloat (f):     
    return float(struct.unpack('<f', struct.pack('<I', f))[0])

def FloatToInt (f):
    return int(struct.unpack('<I', struct.pack('<f', f))[0])

        
class FrameType(IntEnum):
    BroadcastFrame = 0
    SetRegisterResponseFrame = 1
    GetRegisterResponseFrame = 2
    OutgoingIfuFrame = 3
    SetRegisterCommandFrame = 5
    GetRegisterCommandFrame = 6
    IncomingRawFrame = 7
    Invalid = 255
    
class ResponseStatus(IntEnum) :
    Success =                   0x00,
    UnsupportedCommand =        0x01,
    InvalidRegisterAddress =    0x02,
    InvalidIndex =              0x03,
    TypeError =                 0x04,
    DecryptionFailed =          0x05,
    InsufficientAccessLevel =   0x08,
    UnknownError2 =              0xef,
    UnknownError =              0xff


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#==========================================================================
# Harpia REST API Interface
#--------------------------------------------------------------------------
# Copyright (c) 2018 Light Conversion (UAB MGF "Šviesos konversija")
# All rights reserved.
# www.lightcon.com
#==========================================================================

import urllib
import json
import time
from ..common import HTTP_methods

class WinTopas (HTTP_methods):
    silent = True
    connected = False
            
    def __init__ (self, ip_address, serial_number, port=8008, version='v0'):
        self.url = 'http://{}:{}/{}/{}/PublicAPI'.format(ip_address, port, serial_number, version)
            
        self.connected = True;
        if self.connected:
            print ('WinTopas System initialized at', self.url)
        
    def get_optical_system_data(self):
        return self._get(r'/Optical/WavelengthControl/OpticalSystemData')
    
    def set_wavelength(self, wavelength):
        self._put(r'/Optical/WavelengthControl/SetWavelengthUsingAnyInteraction', str(wavelength))
        
    def get_wavelength(self):
        return self._get(r'/Optical/WavelengthControl/Output/Wavelength')
    
    def get_output(self):
        return self._get(r'/Optical/WavelengthControl/Output')
    
    def finish_user_action(self):
        self._put(r'/Optical/WavelengthControl/FinishWavelengthSettingAfterUserActions', r'{"RestoreShutter" :false}')    
        


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#==========================================================================
# Harpia DAQ DLL Wrapper
#--------------------------------------------------------------------------
# Copyright (c) 2020 Light Conversion, UAB
# All rights reserved.
# www.lightcon.com
#==========================================================================

import clr   # pip install pythonnet
import numpy as np
import time

import os
dirname = os.path.dirname(__file__)
clr.AddReference(os.path.join(dirname, "LightConversion.Hardware.HarpiaDaq.dll"))
clr.AddReference(os.path.join(dirname, "LightConversion.Abstractions.dll"))
clr.AddReference(os.path.join(dirname, "System.Threading.dll"))
clr.AddReference("mscorlib.dll")

from LightConversion.Hardware.HarpiaDaq import HarpiaDaq
from LightConversion.Abstractions import ClockChannel, SampleClockActiveEdge, TriggerChannel, DaqException
from System.Threading import CancellationTokenSource, CancellationToken
from System import TimeoutException

class HarpiaDaqWrapper:
    daq = None
    DaqException = DaqException
    TimeoutException = TimeoutException
    def __init__(self):
        devices = HarpiaDaq.Devices
        if len(devices) > 0:
            daq = HarpiaDaq.LoadDevice(devices[0])
            is_sync = ('SDAQ' in daq.Device.ProductType.upper()) or ('SDAQ' in daq.Device.SerialNumber.upper())
            daq.Initialize(is_sync)

            if daq.Connected:
                print ("Initialized", daq.Device.ProductType, daq.Device.SerialNumber)            
                self.daq = daq
                time.sleep(1)
        else:
            print ("Harpia DAQ not found")

    def is_connected(self):
        return self.daq.Connected or False
    
    def enable_channels(self, channels):
        '''
        Enables analog input channels of ADC
        
        Parameters
        ----------
        channels : list
            List of strings, for example ['AI0', 'AI1']
        '''
        self.daq.EnableAIChannels(';'.join(channels))
        
    def set_timeout(self, timeout):
        '''
        Sets timeout in milliseconds
        
        Parameters
        ----------
        timeout : number
            Timeout value, given in milliseconds
        '''
        self.daq.AcquisitionTimeout = int(timeout)
        
    def configure_sample_clock(self, channel, active_edge):
        '''
        Configures clock for ADC acquisition
        
        Parameters
        ----------
        channel : str
            'internal', 'PFI0', 'PFI1', 'PFI2', 'PFI3', 'PFI4' or 'PFI5'
        active_edge : str
            'rising' or 'falling'        
        '''
        my_clock_channel = ClockChannel.Internal
        if (channel == 'internal'):
            my_clock_channel = ClockChannel.Internal
        elif ((channel == 'PFI0')):
            my_clock_channel = ClockChannel.PFI0
        elif ((channel == 'PFI1')):
            my_clock_channel = ClockChannel.PFI1
        elif ((channel == 'PFI2')):
            my_clock_channel = ClockChannel.PFI2
        elif ((channel == 'PFI3')):
            my_clock_channel = ClockChannel.PFI3
        elif ((channel == 'PFI4')):
            my_clock_channel = ClockChannel.PFI4
        elif ((channel == 'PFI5')):
            my_clock_channel = ClockChannel.PFI5
        elif ((channel == 'PFI6')):
            my_clock_channel = ClockChannel.PFI6
            
        my_clock_active_edge = SampleClockActiveEdge.Rising
        if (active_edge == 'rising'):
            my_clock_active_edge = SampleClockActiveEdge.Rising
        else:
            my_clock_active_edge = SampleClockActiveEdge.Falling            
            
        self.daq.ConfigureSampleClock(my_clock_channel, 0.8e6, my_clock_active_edge, 1000)
            
    def configure_start_trigger(self, channel):
        '''
        Configures trigger for ADC acquisition
        
        Parameters
        ----------
        channel : str
            'internal', 'PFI0', 'PFI1', 'PFI2', 'PFI3', 'PFI4' or 'PFI5'
        '''
        my_trigger_channel = TriggerChannel.Internal
        if (channel == 'internal'):
            my_trigger_channel = TriggerChannel.Internal
        elif ((channel == 'PFI0')):
            my_trigger_channel = TriggerChannel.PFI0
        elif ((channel == 'PFI1')):
            my_trigger_channel = TriggerChannel.PFI1
        elif ((channel == 'PFI2')):
            my_trigger_channel = TriggerChannel.PFI2
        elif ((channel == 'PFI3')):
            my_trigger_channel = TriggerChannel.PFI3
        elif ((channel == 'PFI4')):
            my_trigger_channel = TriggerChannel.PFI4
        elif ((channel == 'PFI5')):
            my_trigger_channel = TriggerChannel.PFI5
        elif ((channel == 'PFI6')):
            my_trigger_channel = TriggerChannel.PFI6
        
        self.daq.ConfigureStartTrigger(my_trigger_channel)

    def get_daq_data(self, n):
        '''
        Acquires n samples per enabled channel.
        
        Parameters
        ----------
        n : number
            Number of datapoints to acquire per channel
        
        Returns
        -------
        list (m,n):
            Values in volts, where m is number of enabled channels (in ascending order)
        '''
        self.daq.DatapointsPerChannel = n
        cts = CancellationTokenSource()
        ct = cts.Token

        data = self.daq.GetAnalogChannelsData(ct, True)
        data = [item for item in data]
        return np.reshape(data, (int(len(data)/n), n), order='C')
    
    def close(self):
        self.daq.Dispose()






#==========================================================================
# Komodo Interface Library
#--------------------------------------------------------------------------
# Copyright (c) 2011-2020 Total Phase, Inc.
# All rights reserved.
# www.totalphase.com
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# - Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
#
# - Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
# - Neither the name of Total Phase, Inc. nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#--------------------------------------------------------------------------
# To access Komodo devices through the API:
#
# 1) Use one of the following shared objects:
#      komodo.so      --  Linux shared object
#      komodo.dll     --  Windows dynamic link library
#
# 2) Along with one of the following language modules:
#      komodo.c/h     --  C/C++ API header file and interface module
#      komodo_py.py   --  Python API
#      komodo.bas     --  Visual Basic 6 API
#      komodo.cs      --  C# .NET source
#      komodo_net.dll --  Compiled .NET binding
#==========================================================================


#==========================================================================
# VERSION
#==========================================================================
KM_API_VERSION    = 0x0132   # v1.50
KM_REQ_SW_VERSION = 0x010a   # v1.10


#==========================================================================
# IMPORTS
#==========================================================================
import os
import struct
import sys

from array import array, ArrayType

def import_library ():
    global api

    import platform
    ext = platform.system() == 'Windows' and '.dll' or '.so'
    dir = os.path.dirname(os.path.abspath(__file__))
    lib = os.path.join(dir, 'komodo' + ext)

    try:
        if sys.version_info >= (3, 4):
            from importlib.machinery import ExtensionFileLoader
            from importlib.util import spec_from_file_location
            from importlib.util import module_from_spec

            loader = ExtensionFileLoader('komodo', lib)
            spec = spec_from_file_location('komodo', loader=loader)
            api = module_from_spec(spec)
            spec.loader.exec_module(api)

        else:
            import imp
            api = imp.load_dynamic('komodo', lib)

    except:
        _, err, _ = sys.exc_info()
        msg = 'Error while importing komodo%s:\n%s' % (ext, err)
        sys.exit(msg)

try:
    import komodo as api
except ImportError:
    import_library()

del import_library

KM_SW_VERSION      = api.py_version() & 0xffff
KM_REQ_API_VERSION = (api.py_version() >> 16) & 0xffff
KM_LIBRARY_LOADED  = \
    ((KM_SW_VERSION >= KM_REQ_SW_VERSION) and \
     (KM_API_VERSION >= KM_REQ_API_VERSION))


#==========================================================================
# HELPER FUNCTIONS
#==========================================================================
def array_u08 (n):  return array('B', [0]*n)
def array_u16 (n):  return array('H', [0]*n)
def array_u32 (n):  return array('I', [0]*n)
def array_u64 (n):  return array('K', [0]*n)
def array_s08 (n):  return array('b', [0]*n)
def array_s16 (n):  return array('h', [0]*n)
def array_s32 (n):  return array('i', [0]*n)
def array_s64 (n):  return array('L', [0]*n)
def array_f32 (n):  return array('f', [0]*n)
def array_f64 (n):  return array('d', [0]*n)


#==========================================================================
# STATUS CODES
#==========================================================================
# All API functions return an integer which is the result of the
# transaction, or a status code if negative.  The status codes are
# defined as follows:
# enum km_status_t
# General codes (0 to -99)
KM_OK                      =    0
KM_UNABLE_TO_LOAD_LIBRARY  =   -1
KM_UNABLE_TO_LOAD_DRIVER   =   -2
KM_UNABLE_TO_LOAD_FUNCTION =   -3
KM_INCOMPATIBLE_LIBRARY    =   -4
KM_INCOMPATIBLE_DEVICE     =   -5
KM_COMMUNICATION_ERROR     =   -6
KM_UNABLE_TO_OPEN          =   -7
KM_UNABLE_TO_CLOSE         =   -8
KM_INVALID_HANDLE          =   -9
KM_CONFIG_ERROR            =  -10
KM_PARAM_ERROR             =  -11
KM_FUNCTION_NOT_AVAILABLE  =  -12
KM_FEATURE_NOT_ACQUIRED    =  -13
KM_NOT_DISABLED            =  -14
KM_NOT_ENABLED             =  -15

# CAN codes (-100 to -199)
KM_CAN_READ_EMPTY          = -101
KM_CAN_SEND_TIMEOUT        = -102
KM_CAN_SEND_FAIL           = -103
KM_CAN_ASYNC_EMPTY         = -104
KM_CAN_ASYNC_MAX_REACHED   = -105
KM_CAN_ASYNC_PENDING       = -106
KM_CAN_ASYNC_TIMEOUT       = -107
KM_CAN_AUTO_BITRATE_FAIL   = -108


#==========================================================================
# GENERAL TYPE DEFINITIONS
#==========================================================================
# Komodo handle type definition
# typedef Komodo => integer

# Komodo version matrix.
#
# This matrix describes the various version dependencies
# of Komodo components.  It can be used to determine
# which component caused an incompatibility error.
#
# All version numbers are of the format:
#   (major << 8) | minor
#
# ex. v1.20 would be encoded as:  0x0114
class KomodoVersion:
    def __init__ (self):
        # Software, firmware, and hardware versions.
        self.software       = 0
        self.firmware       = 0
        self.hardware       = 0

        # Firmware revisions that are compatible with this software version.
        # The top 16 bits gives the maximum accepted fw revision.
        # The lower 16 bits gives the minimum accepted fw revision.
        self.fw_revs_for_sw = 0

        # Hardware revisions that are compatible with this software version.
        # The top 16 bits gives the maximum accepted hw revision.
        # The lower 16 bits gives the minimum accepted hw revision.
        self.hw_revs_for_sw = 0

        # Software requires that the API interface must be >= this version.
        self.api_req_by_sw  = 0

# Komodo feature set
#
# This bitmask field describes the features available on this device.
#
# When returned by km_features() or km_open_ext(), it refers to the
# potential features of the device.
# When used as a parameter by km_enable() or km_disable(), it refers
# to the features that the user wants to use.
# And when returned by km_disable(), it refers to the features currently
# in use by the user.
KM_FEATURE_GPIO_LISTEN = 0x00000001
KM_FEATURE_GPIO_CONTROL = 0x00000002
KM_FEATURE_GPIO_CONFIG = 0x00000004
KM_FEATURE_CAN_A_LISTEN = 0x00000008
KM_FEATURE_CAN_A_CONTROL = 0x00000010
KM_FEATURE_CAN_A_CONFIG = 0x00000020
KM_FEATURE_CAN_B_LISTEN = 0x00000040
KM_FEATURE_CAN_B_CONTROL = 0x00000080
KM_FEATURE_CAN_B_CONFIG = 0x00000100

#==========================================================================
# GENERAL API
#==========================================================================
# Get a list of ports to which Komodo devices are attached.
#
# nelem   = maximum number of elements to return
# devices = array into which the port numbers are returned
#
# Each element of the array is written with the port number.
# Devices that are in-use are ORed with KM_PORT_NOT_FREE (0x8000).
#
# ex.  devices are attached to ports 0, 1, 2
#      ports 0 and 2 are available, and port 1 is in-use.
#      array => 0x0000, 0x8001, 0x0002
#
# If the array is NULL, it is not filled with any values.
# If there are more devices than the array size, only the
# first nmemb port numbers will be written into the array.
#
# Returns the number of devices found, regardless of the
# array size.
KM_PORT_NOT_FREE = 0x8000
KM_PORT_NUM_MASK = 0x00ff
def km_find_devices (ports):
    """usage: (int return, u16[] ports) = km_find_devices(u16[] ports)

    All arrays can be passed into the API as an ArrayType object or as
    a tuple (array, length), where array is an ArrayType object and
    length is an integer.  The user-specified length would then serve
    as the length argument to the API funtion (please refer to the
    product datasheet).  If only the array is provided, the array's
    intrinsic length is used as the argument to the underlying API
    function.

    Additionally, for arrays that are filled by the API function, an
    integer can be passed in place of the array argument and the API
    will automatically create an array of that length.  All output
    arrays, whether passed in or generated, are passed back in the
    returned tuple."""

    if not KM_LIBRARY_LOADED: return KM_INCOMPATIBLE_LIBRARY
    # ports pre-processing
    __ports = isinstance(ports, int)
    if __ports:
        (ports, num_ports) = (array_u16(ports), ports)
    else:
        (ports, num_ports) = isinstance(ports, ArrayType) and (ports, len(ports)) or (ports[0], min(len(ports[0]), int(ports[1])))
        if ports.typecode != 'H':
            raise TypeError("type for 'ports' must be array('H')")
    # Call API function
    (_ret_) = api.py_km_find_devices(num_ports, ports)
    # ports post-processing
    if __ports: del ports[max(0, min(_ret_, len(ports))):]
    return (_ret_, ports)


# Get a list of ports to which Komodo devices are attached.
#
# This function is the same as km_find_devices() except that
# it returns the unique IDs of each Komodo device.  The IDs
# are guaranteed to be non-zero if valid.
#
# The IDs are the unsigned integer representation of the 10-digit
# serial numbers.
def km_find_devices_ext (ports, unique_ids):
    """usage: (int return, u16[] ports, u32[] unique_ids) = km_find_devices_ext(u16[] ports, u32[] unique_ids)

    All arrays can be passed into the API as an ArrayType object or as
    a tuple (array, length), where array is an ArrayType object and
    length is an integer.  The user-specified length would then serve
    as the length argument to the API funtion (please refer to the
    product datasheet).  If only the array is provided, the array's
    intrinsic length is used as the argument to the underlying API
    function.

    Additionally, for arrays that are filled by the API function, an
    integer can be passed in place of the array argument and the API
    will automatically create an array of that length.  All output
    arrays, whether passed in or generated, are passed back in the
    returned tuple."""

    if not KM_LIBRARY_LOADED: return KM_INCOMPATIBLE_LIBRARY
    # ports pre-processing
    __ports = isinstance(ports, int)
    if __ports:
        (ports, num_ports) = (array_u16(ports), ports)
    else:
        (ports, num_ports) = isinstance(ports, ArrayType) and (ports, len(ports)) or (ports[0], min(len(ports[0]), int(ports[1])))
        if ports.typecode != 'H':
            raise TypeError("type for 'ports' must be array('H')")
    # unique_ids pre-processing
    __unique_ids = isinstance(unique_ids, int)
    if __unique_ids:
        (unique_ids, num_ids) = (array_u32(unique_ids), unique_ids)
    else:
        (unique_ids, num_ids) = isinstance(unique_ids, ArrayType) and (unique_ids, len(unique_ids)) or (unique_ids[0], min(len(unique_ids[0]), int(unique_ids[1])))
        if unique_ids.typecode != 'I':
            raise TypeError("type for 'unique_ids' must be array('I')")
    # Call API function
    (_ret_) = api.py_km_find_devices_ext(num_ports, num_ids, ports, unique_ids)
    # ports post-processing
    if __ports: del ports[max(0, min(_ret_, len(ports))):]
    # unique_ids post-processing
    if __unique_ids: del unique_ids[max(0, min(_ret_, len(unique_ids))):]
    return (_ret_, ports, unique_ids)


# Open the Komodo port.
#
# The port number is a zero-indexed integer.
#
# The port number is the same as that obtained from the
# km_find_devices() function above.
#
# Returns an Komodo handle, which is guaranteed to be
# greater than zero if it is valid.
#
# This function is recommended for use in simple applications
# where extended information is not required.  For more complex
# applications, the use of km_open_ext() is recommended.
def km_open (port_number):
    """usage: Komodo return = km_open(int port_number)"""

    if not KM_LIBRARY_LOADED: return KM_INCOMPATIBLE_LIBRARY
    # Call API function
    return api.py_km_open(port_number)


# Open the Komodo port, returning extended information
# in the supplied structure.  Behavior is otherwise identical
# to km_open() above.  If 0 is passed as the pointer to the
# structure, this function is exactly equivalent to km_open().
#
# The structure is zeroed before the open is attempted.
# It is filled with whatever information is available.
#
# For example, if the firmware version is not filled, then
# the device could not be queried for its version number.
#
# The feature list is a bitmap of Komodo resources, with the same
# mapping as obtained from the km_features() function below.
# Details on the bitmask are found above.
#
# This function is recommended for use in complex applications
# where extended information is required.  For more simple
# applications, the use of km_open() is recommended.
class KomodoExt:
    def __init__ (self):
        # Version matrix
        self.version  = KomodoVersion()

        # Features of this device.
        self.features = 0

def km_open_ext (port_number):
    """usage: (Komodo return, KomodoExt km_ext) = km_open_ext(int port_number)"""

    if not KM_LIBRARY_LOADED: return KM_INCOMPATIBLE_LIBRARY
    # Call API function
    (_ret_, c_km_ext) = api.py_km_open_ext(port_number)
    # km_ext post-processing
    km_ext = KomodoExt()
    (km_ext.version.software, km_ext.version.firmware, km_ext.version.hardware, km_ext.version.fw_revs_for_sw, km_ext.version.hw_revs_for_sw, km_ext.version.api_req_by_sw, km_ext.features) = c_km_ext
    return (_ret_, km_ext)


# Close the Komodo port.
def km_close (komodo):
    """usage: int return = km_close(Komodo komodo)"""

    if not KM_LIBRARY_LOADED: return KM_INCOMPATIBLE_LIBRARY
    # Call API function
    return api.py_km_close(komodo)


# Return the port for this Komodo handle.
#
# The port number is a zero-indexed integer identical to those
# returned by km_find_devices() above.  This includes the count of
# interfaces in use in the upper byte.
def km_port (komodo):
    """usage: int return = km_port(Komodo komodo)"""

    if not KM_LIBRARY_LOADED: return KM_INCOMPATIBLE_LIBRARY
    # Call API function
    return api.py_km_port(komodo)


# Return the device features as a bit-mask of values, or an error code
# if the handle is not valid.  Details on the bitmask are found above.
def km_features (komodo):
    """usage: int return = km_features(Komodo komodo)"""

    if not KM_LIBRARY_LOADED: return KM_INCOMPATIBLE_LIBRARY
    # Call API function
    return api.py_km_features(komodo)


# Return the unique ID for this Komodo adapter.
# IDs are guaranteed to be non-zero if valid.
# The ID is the unsigned integer representation of the
# 10-digit serial number.
def km_unique_id (komodo):
    """usage: u32 return = km_unique_id(Komodo komodo)"""

    if not KM_LIBRARY_LOADED: return KM_INCOMPATIBLE_LIBRARY
    # Call API function
    return api.py_km_unique_id(komodo)


# Return the status string for the given status code.
# If the code is not valid or the library function cannot
# be loaded, return a NULL string.
def km_status_string (status):
    """usage: str return = km_status_string(int status)"""

    if not KM_LIBRARY_LOADED: return KM_INCOMPATIBLE_LIBRARY
    # Call API function
    return api.py_km_status_string(status)


# Return the version matrix for the device attached to the
# given handle.  If the handle is 0 or invalid, only the
# software and required api versions are set.
def km_version (komodo):
    """usage: (int return, KomodoVersion version) = km_version(Komodo komodo)"""

    if not KM_LIBRARY_LOADED: return KM_INCOMPATIBLE_LIBRARY
    # Call API function
    (_ret_, c_version) = api.py_km_version(komodo)
    # version post-processing
    version = KomodoVersion()
    (version.software, version.firmware, version.hardware, version.fw_revs_for_sw, version.hw_revs_for_sw, version.api_req_by_sw) = c_version
    return (_ret_, version)


# Sleep for the specified number of milliseconds.
# Accuracy depends on the operating system scheduler.
# Returns the number of milliseconds slept.
def km_sleep_ms (milliseconds):
    """usage: u32 return = km_sleep_ms(u32 milliseconds)"""

    if not KM_LIBRARY_LOADED: return KM_INCOMPATIBLE_LIBRARY
    # Call API function
    return api.py_km_sleep_ms(milliseconds)


# Acquire device features.
# Returns the features that are currently acquired.
def km_acquire (komodo, features):
    """usage: int return = km_acquire(Komodo komodo, u32 features)"""

    if not KM_LIBRARY_LOADED: return KM_INCOMPATIBLE_LIBRARY
    # Call API function
    return api.py_km_acquire(komodo, features)


# Release device features.
# Returns the features that are still acquired.
def km_release (komodo, features):
    """usage: int return = km_release(Komodo komodo, u32 features)"""

    if not KM_LIBRARY_LOADED: return KM_INCOMPATIBLE_LIBRARY
    # Call API function
    return api.py_km_release(komodo, features)



#==========================================================================
# CAN API
#==========================================================================
# These special timeout constants can be used with the functions
# km_timeout and km_can_async_collect.
KM_TIMEOUT_IMMEDIATE = 0
KM_TIMEOUT_INFINITE = -1
# Set the timeout of the km_can_read function to the specified
# number of milliseconds.
def km_timeout (komodo, timeout_ms):
    """usage: int return = km_timeout(Komodo komodo, u32 timeout_ms)"""

    if not KM_LIBRARY_LOADED: return KM_INCOMPATIBLE_LIBRARY
    # Call API function
    return api.py_km_timeout(komodo, timeout_ms)


# Standard enumeration for the CAN channels available on the Komodo.
# enum km_can_ch_t
KM_CAN_CH_A = 0
KM_CAN_CH_B = 1

# CAN Bus state constants.
KM_CAN_BUS_STATE_LISTEN_ONLY = 0x00000001
KM_CAN_BUS_STATE_CONTROL = 0x00000002
KM_CAN_BUS_STATE_WARNING = 0x00000004
KM_CAN_BUS_STATE_ACTIVE = 0x00000008
KM_CAN_BUS_STATE_PASSIVE = 0x00000010
KM_CAN_BUS_STATE_OFF = 0x00000020
# Retreive the current bus state of the supplied CAN channel
def km_can_query_bus_state (komodo, channel):
    """usage: (int return, u08 bus_state, u08 rx_error, u08 tx_error) = km_can_query_bus_state(Komodo komodo, km_can_ch_t channel)"""

    if not KM_LIBRARY_LOADED: return KM_INCOMPATIBLE_LIBRARY
    # Call API function
    return api.py_km_can_query_bus_state(komodo, channel)


# Set the capture latency to the specified number of milliseconds.
# This number determines the minimum time that a read call will
# block if there is no available data.  Lower times result in
# faster turnaround at the expense of reduced buffering.  Setting
# this parameter too low can cause packets to be dropped.
def km_latency (komodo, latency_ms):
    """usage: int return = km_latency(Komodo komodo, u32 latency_ms)"""

    if not KM_LIBRARY_LOADED: return KM_INCOMPATIBLE_LIBRARY
    # Call API function
    return api.py_km_latency(komodo, latency_ms)


# Config mask for km_can_configure
KM_CAN_CONFIG_NONE = 0x00000000
KM_CAN_CONFIG_LISTEN_SELF = 0x00000001
def km_can_configure (komodo, config):
    """usage: int return = km_can_configure(Komodo komodo, u32 config)"""

    if not KM_LIBRARY_LOADED: return KM_INCOMPATIBLE_LIBRARY
    # Call API function
    return api.py_km_can_configure(komodo, config)


# Set the bus timeout.  If a zero is passed as the timeout,
# the timeout is unchanged and the current timeout is returned.
def km_can_bus_timeout (komodo, channel, timeout_ms):
    """usage: int return = km_can_bus_timeout(Komodo komodo, km_can_ch_t channel, u16 timeout_ms)"""

    if not KM_LIBRARY_LOADED: return KM_INCOMPATIBLE_LIBRARY
    # Call API function
    return api.py_km_can_bus_timeout(komodo, channel, timeout_ms)


# Set the CAN bit rate in hertz on the given channel.  If a zero is
# passed as the bitrate, the bitrate is unchanged.  In all cases, the
# call will return the bitrate that will be in effect.
KM_KHZ = 1000
KM_MHZ = 1000000
def km_can_bitrate (komodo, channel, bitrate_hz):
    """usage: int return = km_can_bitrate(Komodo komodo, km_can_ch_t channel, u32 bitrate_hz)"""

    if not KM_LIBRARY_LOADED: return KM_INCOMPATIBLE_LIBRARY
    # Call API function
    return api.py_km_can_bitrate(komodo, channel, bitrate_hz)


def km_can_auto_bitrate (komodo, channel):
    """usage: int return = km_can_auto_bitrate(Komodo komodo, km_can_ch_t channel)"""

    if not KM_LIBRARY_LOADED: return KM_INCOMPATIBLE_LIBRARY
    # Call API function
    return api.py_km_can_auto_bitrate(komodo, channel)


def km_can_auto_bitrate_ext (komodo, channel, bitrates_hz):
    """usage: int return = km_can_auto_bitrate_ext(Komodo komodo, km_can_ch_t channel, u32[] bitrates_hz)

    All arrays can be passed into the API as an ArrayType object or as
    a tuple (array, length), where array is an ArrayType object and
    length is an integer.  The user-specified length would then serve
    as the length argument to the API funtion (please refer to the
    product datasheet).  If only the array is provided, the array's
    intrinsic length is used as the argument to the underlying API
    function."""

    if not KM_LIBRARY_LOADED: return KM_INCOMPATIBLE_LIBRARY
    # bitrates_hz pre-processing
    (bitrates_hz, num_bitrates_hz) = isinstance(bitrates_hz, ArrayType) and (bitrates_hz, len(bitrates_hz)) or (bitrates_hz[0], min(len(bitrates_hz[0]), int(bitrates_hz[1])))
    if bitrates_hz.typecode != 'I':
        raise TypeError("type for 'bitrates_hz' must be array('I')")
    # Call API function
    return api.py_km_can_auto_bitrate_ext(komodo, channel, num_bitrates_hz, bitrates_hz)


# Get the sample rate in hertz.
def km_get_samplerate (komodo):
    """usage: int return = km_get_samplerate(Komodo komodo)"""

    if not KM_LIBRARY_LOADED: return KM_INCOMPATIBLE_LIBRARY
    # Call API function
    return api.py_km_get_samplerate(komodo)


# Configure the target power.  Returns power status or error code.
# enum km_power_t
KM_TARGET_POWER_QUERY = 0x02
KM_TARGET_POWER_OFF   = 0x00
KM_TARGET_POWER_ON    = 0x01

# Set the target power for the specified CAN channel.
def km_can_target_power (komodo, channel, power):
    """usage: int return = km_can_target_power(Komodo komodo, km_can_ch_t channel, km_power_t power)"""

    if not KM_LIBRARY_LOADED: return KM_INCOMPATIBLE_LIBRARY
    # Call API function
    return api.py_km_can_target_power(komodo, channel, power)


# Enable the Komodo.
def km_enable (komodo):
    """usage: int return = km_enable(Komodo komodo)"""

    if not KM_LIBRARY_LOADED: return KM_INCOMPATIBLE_LIBRARY
    # Call API function
    return api.py_km_enable(komodo)


# Disable the Komodo.
def km_disable (komodo):
    """usage: int return = km_disable(Komodo komodo)"""

    if not KM_LIBRARY_LOADED: return KM_INCOMPATIBLE_LIBRARY
    # Call API function
    return api.py_km_disable(komodo)


# Status mask for km_can_read
KM_READ_TIMEOUT = 0x80000000
KM_READ_ERR_OVERFLOW = 0x40000000
KM_READ_END_OF_CAPTURE = 0x20000000
KM_READ_CAN_ERR = 0x00000100
KM_READ_CAN_ERR_FULL_MASK = 0x000000ff
KM_READ_CAN_ERR_POS_MASK = 0x0000001f
KM_READ_CAN_ERR_POS_SOF = 0x00000003
KM_READ_CAN_ERR_POS_ID28_21 = 0x00000002
KM_READ_CAN_ERR_POS_ID20_18 = 0x00000006
KM_READ_CAN_ERR_POS_SRTR = 0x00000004
KM_READ_CAN_ERR_POS_IDE = 0x00000005
KM_READ_CAN_ERR_POS_ID17_13 = 0x00000007
KM_READ_CAN_ERR_POS_ID12_5 = 0x0000000f
KM_READ_CAN_ERR_POS_ID4_0 = 0x0000000e
KM_READ_CAN_ERR_POS_RTR = 0x0000000c
KM_READ_CAN_ERR_POS_RSVD_1 = 0x0000000d
KM_READ_CAN_ERR_POS_RSVD_0 = 0x00000009
KM_READ_CAN_ERR_POS_DLC = 0x0000000b
KM_READ_CAN_ERR_POS_DF = 0x0000000a
KM_READ_CAN_ERR_POS_CRC_SEQ = 0x00000008
KM_READ_CAN_ERR_POS_CRC_DEL = 0x00000018
KM_READ_CAN_ERR_POS_ACK_SLOT = 0x00000019
KM_READ_CAN_ERR_POS_ACK_DEL = 0x0000001b
KM_READ_CAN_ERR_POS_EOF = 0x0000001a
KM_READ_CAN_ERR_POS_INTRMSN = 0x00000012
KM_READ_CAN_ERR_POS_AEF = 0x00000011
KM_READ_CAN_ERR_POS_PEF = 0x00000016
KM_READ_CAN_ERR_POS_TDB = 0x00000013
KM_READ_CAN_ERR_POS_ERR_DEL = 0x00000017
KM_READ_CAN_ERR_POS_OVRFLG = 0x0000001c
KM_READ_CAN_ERR_DIR_MASK = 0x00000020
KM_READ_CAN_ERR_DIR_TX = 0x00000000
KM_READ_CAN_ERR_DIR_RX = 0x00000020
KM_READ_CAN_ERR_TYPE_MASK = 0x000000c0
KM_READ_CAN_ERR_TYPE_BIT = 0x00000000
KM_READ_CAN_ERR_TYPE_FORM = 0x00000040
KM_READ_CAN_ERR_TYPE_STUFF = 0x00000080
KM_READ_CAN_ERR_TYPE_OTHER = 0x000000c0
KM_READ_CAN_ARB_LOST = 0x00000200
KM_READ_CAN_ARB_LOST_POS_MASK = 0x000000ff
# GPIO Configuration
KM_GPIO_PIN_1_CONFIG = 0x00
KM_GPIO_PIN_2_CONFIG = 0x01
KM_GPIO_PIN_3_CONFIG = 0x02
KM_GPIO_PIN_4_CONFIG = 0x03
KM_GPIO_PIN_5_CONFIG = 0x04
KM_GPIO_PIN_6_CONFIG = 0x05
KM_GPIO_PIN_7_CONFIG = 0x06
KM_GPIO_PIN_8_CONFIG = 0x07
# GPIO Mask
KM_GPIO_PIN_1_MASK = 0x01
KM_GPIO_PIN_2_MASK = 0x02
KM_GPIO_PIN_3_MASK = 0x04
KM_GPIO_PIN_4_MASK = 0x08
KM_GPIO_PIN_5_MASK = 0x10
KM_GPIO_PIN_6_MASK = 0x20
KM_GPIO_PIN_7_MASK = 0x40
KM_GPIO_PIN_8_MASK = 0x80
# Event mask for km_can_read
KM_EVENT_DIGITAL_INPUT = 0x00000100
KM_EVENT_DIGITAL_INPUT_MASK = 0x000000ff
KM_EVENT_DIGITAL_INPUT_1 = 0x00000001
KM_EVENT_DIGITAL_INPUT_2 = 0x00000002
KM_EVENT_DIGITAL_INPUT_3 = 0x00000004
KM_EVENT_DIGITAL_INPUT_4 = 0x00000008
KM_EVENT_DIGITAL_INPUT_5 = 0x00000010
KM_EVENT_DIGITAL_INPUT_6 = 0x00000020
KM_EVENT_DIGITAL_INPUT_7 = 0x00000040
KM_EVENT_DIGITAL_INPUT_8 = 0x00000080
KM_EVENT_CAN_BUS_STATE_LISTEN_ONLY = 0x00001000
KM_EVENT_CAN_BUS_STATE_CONTROL = 0x00002000
KM_EVENT_CAN_BUS_STATE_WARNING = 0x00004000
KM_EVENT_CAN_BUS_STATE_ACTIVE = 0x00008000
KM_EVENT_CAN_BUS_STATE_PASSIVE = 0x00010000
KM_EVENT_CAN_BUS_STATE_OFF = 0x00020000
KM_EVENT_CAN_BUS_BITRATE = 0x00040000
class km_can_info_t:
    def __init__ (self):
        self.timestamp      = 0
        self.status         = 0
        self.events         = 0
        self.channel        = 0
        self.bitrate_hz     = 0
        self.host_gen       = 0
        self.rx_error_count = 0
        self.tx_error_count = 0
        self.overflow_count = 0

class km_can_packet_t:
    def __init__ (self):
        self.remote_req  = 0
        self.extend_addr = 0
        self.dlc         = 0
        self.id          = 0

# Read a single CAN packet from the Komodo data stream.
# This will block for timeout_ms milliseconds; 0 will return
# immediately, and MAXINT will block indefinitely.
# timestamp is in units of nanoseconds.
def km_can_read (komodo, data):
    """usage: (int return, km_can_info_t info, km_can_packet_t pkt, u08[] data) = km_can_read(Komodo komodo, u08[] data)

    All arrays can be passed into the API as an ArrayType object or as
    a tuple (array, length), where array is an ArrayType object and
    length is an integer.  The user-specified length would then serve
    as the length argument to the API funtion (please refer to the
    product datasheet).  If only the array is provided, the array's
    intrinsic length is used as the argument to the underlying API
    function.

    Additionally, for arrays that are filled by the API function, an
    integer can be passed in place of the array argument and the API
    will automatically create an array of that length.  All output
    arrays, whether passed in or generated, are passed back in the
    returned tuple."""

    if not KM_LIBRARY_LOADED: return KM_INCOMPATIBLE_LIBRARY
    # data pre-processing
    __data = isinstance(data, int)
    if __data:
        (data, num_bytes) = (array_u08(data), data)
    else:
        (data, num_bytes) = isinstance(data, ArrayType) and (data, len(data)) or (data[0], min(len(data[0]), int(data[1])))
        if data.typecode != 'B':
            raise TypeError("type for 'data' must be array('B')")
    # Call API function
    (_ret_, c_info, c_pkt) = api.py_km_can_read(komodo, num_bytes, data)
    # info post-processing
    info = km_can_info_t()
    (info.timestamp, info.status, info.events, info.channel, info.bitrate_hz, info.host_gen, info.rx_error_count, info.tx_error_count, info.overflow_count) = c_info
    # pkt post-processing
    pkt = km_can_packet_t()
    (pkt.remote_req, pkt.extend_addr, pkt.dlc, pkt.id) = c_pkt
    # data post-processing
    if __data: del data[max(0, min(_ret_, len(data))):]
    return (_ret_, info, pkt, data)


# Flags mask
KM_CAN_ONE_SHOT = 0x01
# Submit a CAN packet to the Komodo data stream, asynchronously.
def km_can_async_submit (komodo, channel, flags, pkt, data):
    """usage: int return = km_can_async_submit(Komodo komodo, km_can_ch_t channel, u08 flags, km_can_packet_t pkt, u08[] data)

    All arrays can be passed into the API as an ArrayType object or as
    a tuple (array, length), where array is an ArrayType object and
    length is an integer.  The user-specified length would then serve
    as the length argument to the API funtion (please refer to the
    product datasheet).  If only the array is provided, the array's
    intrinsic length is used as the argument to the underlying API
    function."""

    if not KM_LIBRARY_LOADED: return KM_INCOMPATIBLE_LIBRARY
    # pkt pre-processing
    c_pkt = None
    if pkt != None:
        c_pkt = (pkt.remote_req, pkt.extend_addr, pkt.dlc, pkt.id)
    # data pre-processing
    (data, num_bytes) = isinstance(data, ArrayType) and (data, len(data)) or (data[0], min(len(data[0]), int(data[1])))
    if data.typecode != 'B':
        raise TypeError("type for 'data' must be array('B')")
    # Call API function
    return api.py_km_can_async_submit(komodo, channel, flags, c_pkt, num_bytes, data)


# Collect a response to a CAN packet submitted to the Komodo data
# stream, asynchronously.
def km_can_async_collect (komodo, timeout_ms):
    """usage: (int return, u32 arbitration_count) = km_can_async_collect(Komodo komodo, u32 timeout_ms)"""

    if not KM_LIBRARY_LOADED: return KM_INCOMPATIBLE_LIBRARY
    # Call API function
    return api.py_km_can_async_collect(komodo, timeout_ms)


# Write a stream of bytes to the CAN slave device.  The return
# value of the function is a status code.
def km_can_write (komodo, channel, flags, pkt, data):
    """usage: (int return, u32 arbitration_count) = km_can_write(Komodo komodo, km_can_ch_t channel, u08 flags, km_can_packet_t pkt, u08[] data)

    All arrays can be passed into the API as an ArrayType object or as
    a tuple (array, length), where array is an ArrayType object and
    length is an integer.  The user-specified length would then serve
    as the length argument to the API funtion (please refer to the
    product datasheet).  If only the array is provided, the array's
    intrinsic length is used as the argument to the underlying API
    function."""

    if not KM_LIBRARY_LOADED: return KM_INCOMPATIBLE_LIBRARY
    # pkt pre-processing
    c_pkt = None
    if pkt != None:
        c_pkt = (pkt.remote_req, pkt.extend_addr, pkt.dlc, pkt.id)
    # data pre-processing
    (data, num_bytes) = isinstance(data, ArrayType) and (data, len(data)) or (data[0], min(len(data[0]), int(data[1])))
    if data.typecode != 'B':
        raise TypeError("type for 'data' must be array('B')")
    # Call API function
    return api.py_km_can_write(komodo, channel, flags, c_pkt, num_bytes, data)



#==========================================================================
# GPIO API
#==========================================================================
# Enumeration of input GPIO pin bias configurations.
# enum km_pin_bias_t
KM_PIN_BIAS_TRISTATE = 0x00
KM_PIN_BIAS_PULLUP   = 0x01
KM_PIN_BIAS_PULLDOWN = 0x02

# Enumeration of input GPIO pin trigger edge condition.
# enum km_pin_trigger_t
KM_PIN_TRIGGER_NONE         = 0x00
KM_PIN_TRIGGER_RISING_EDGE  = 0x01
KM_PIN_TRIGGER_FALLING_EDGE = 0x02
KM_PIN_TRIGGER_BOTH_EDGES   = 0x03

# Configure a GPIO pin to act as an input.  The return value
# of the function is a status code
def km_gpio_config_in (komodo, pin_number, bias, trigger):
    """usage: int return = km_gpio_config_in(Komodo komodo, u08 pin_number, u08 bias, u08 trigger)"""

    if not KM_LIBRARY_LOADED: return KM_INCOMPATIBLE_LIBRARY
    # Call API function
    return api.py_km_gpio_config_in(komodo, pin_number, bias, trigger)


# Enumeration of output GPIO pin driver configurations.
# enum km_pin_drive_t
KM_PIN_DRIVE_NORMAL            = 0x00
KM_PIN_DRIVE_INVERTED          = 0x01
KM_PIN_DRIVE_OPEN_DRAIN        = 0x02
KM_PIN_DRIVE_OPEN_DRAIN_PULLUP = 0x03

# Enumeration of output GPIO pin sources.
# enum km_pin_source_t
KM_PIN_SRC_SOFTWARE_CTL       = 0x00
KM_PIN_SRC_ALL_ERR_CAN_A      = 0x11
KM_PIN_SRC_BIT_ERR_CAN_A      = 0x12
KM_PIN_SRC_FORM_ERR_CAN_A     = 0x13
KM_PIN_SRC_STUFF_ERR_CAN_A    = 0x14
KM_PIN_SRC_OTHER_ERR_CAN_A    = 0x15
KM_PIN_SRC_ALL_ERR_CAN_B      = 0x21
KM_PIN_SRC_BIT_ERR_CAN_B      = 0x22
KM_PIN_SRC_FORM_ERR_CAN_B     = 0x23
KM_PIN_SRC_STUFF_ERR_CAN_B    = 0x24
KM_PIN_SRC_OTHER_ERR_CAN_B    = 0x25
KM_PIN_SRC_ALL_ERR_CAN_BOTH   = 0x31
KM_PIN_SRC_BIT_ERR_CAN_BOTH   = 0x32
KM_PIN_SRC_FORM_ERR_CAN_BOTH  = 0x33
KM_PIN_SRC_STUFF_ERR_CAN_BOTH = 0x34
KM_PIN_SRC_OTHER_ERR_CAN_BOTH = 0x35

# Configure a GPIO pin to act as an output. The return value
# of the function is a status code.
def km_gpio_config_out (komodo, pin_number, drive, source):
    """usage: int return = km_gpio_config_out(Komodo komodo, u08 pin_number, u08 drive, u08 source)"""

    if not KM_LIBRARY_LOADED: return KM_INCOMPATIBLE_LIBRARY
    # Call API function
    return api.py_km_gpio_config_out(komodo, pin_number, drive, source)


# Set the value of any GPIO pins configured as software controlled
# outputs. The return value of the function is a status code.
def km_gpio_set (komodo, value, mask):
    """usage: int return = km_gpio_set(Komodo komodo, u08 value, u08 mask)"""

    if not KM_LIBRARY_LOADED: return KM_INCOMPATIBLE_LIBRARY
    # Call API function
    return api.py_km_gpio_set(komodo, value, mask)


# Returns the current values of all GPIO pins.
def km_gpio_get (komodo):
    """usage: int return = km_gpio_get(Komodo komodo)"""

    if not KM_LIBRARY_LOADED: return KM_INCOMPATIBLE_LIBRARY
    # Call API function
    return api.py_km_gpio_get(komodo)





# -*- coding: utf-8 -*-
"""
Created on Wed Feb  5 17:21:57 2020

@author: butkus
"""

import numpy as np

def motor_parameters_for_L6470(R, Ir, L, Ke, Vbus=24.0, lsopt=True):
    '''Calculates optimal parameter for L6470 driver using given values
    of resistance R [Ohm], rated current Ir [A], inductivity L [H], phase voltage constant Ke [V/Hz]
    and bus voltage Vbus [V]. lsopt=True if low-speed optimization used'''
    KVAL = int(R * Ir * np.sqrt(2.0) / Vbus * (2.0 ** 8))
    FN_SLP = int((2.0 * np.pi * L * Ir * np.sqrt(2.0) + Ke) / 4.0 / Vbus * (2 ** 16))
    INT_SPEED = int(4.0 * R / (2.0 * np.pi * L) * (2**26) * 250.0E-9)
    pars = { 'KVAL_HOLD': KVAL >> 1,
            'KVAL_RUN': KVAL,
            'KVAL_ACC': KVAL,
            'KVAL_DEC': KVAL,
            'INT_SPEED': int(2**14-1) if (INT_SPEED > 2**14 - 1) else INT_SPEED,
            'ST_SLP': int(Ke / 4.0 / Vbus * (2**16)),
            'FN_SLP_ACC': FN_SLP,
            'FN_SLP_DEC': FN_SLP,
            'MIN_SPEED': int(50.0 * (2**24) * 250.0E-9) | ((1 << 12) if lsopt else 0 )            
            }
    
    return pars
    



#!/usr/bin/env python3
#==========================================================================
# (c) 2011-2019  Total Phase, Inc.
#--------------------------------------------------------------------------
# Project : Komodo Examples
# File    : loopback.py
#--------------------------------------------------------------------------
# Demonstrates how to open ports, acquire features, write and read data.
#--------------------------------------------------------------------------
# Redistribution and use of this file in source and binary forms, with
# or without modification, are permitted.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#========================================================================*/

#==========================================================================
# IMPORTS
#========================================================================*/
from __future__ import division, with_statement, print_function
from .komodo_py import *

class Komodo:
    port = 0
    timeout = 1000 #ms
    bitrate = 1000000 #Hz
    
    def __init__ (self, port = 0, timeout = 1000, bitrate = 1000000):
        self.port = port
        self.timeout = timeout
        self.bitrate = bitrate
    
    def connect(self):
        # Connect to and configure
        self.kma = km_open(self.port)
        
        # Setup CAN channel A for writing and B for listening
        reta = km_acquire(self.kma, KM_FEATURE_CAN_A_CONFIG  |
                               KM_FEATURE_CAN_A_CONTROL |
                               KM_FEATURE_CAN_A_LISTEN)
        
        # Set bitrate
        reta = km_can_bitrate(self.kma, KM_CAN_CH_A, self.bitrate)
        
        # Set timeout
        km_timeout(self.kma, self.timeout)
        
        # Set target power
        km_can_target_power(self.kma, KM_CAN_CH_A, KM_TARGET_POWER_ON)
        
        km_enable(self.kma)

    def close(self):
        km_disable(self.kma)
        km_can_target_power(self.kma, KM_CAN_CH_A, KM_TARGET_POWER_OFF)
    
        # Close and exit
        km_close(self.kma)
    
    def send(self, can_id, data):
        info      = km_can_info_t()
        data_out  = array('B', data)
        data_in   = array('B', [0]*len(data_out))
        
        pkt       = km_can_packet_t()
        pkt.dlc   = len(data_out)
        pkt.id    = can_id
        
        km_can_write(self.kma, KM_CAN_CH_A, 0, pkt, data_out)
    
    def receive(self):
        data_in = array('B', [0]*8)
        ret = 0
        while(ret == 0):
            (ret, info, pkt, data_in) = km_can_read(self.kma, data_in)
            
        return list(data_in)
##==========================================================================
## MAIN PROGRAM
##==========================================================================
#base_id = 0x190
#
#komodo_open()
#
##komodo_send(base_id + 5, [0x01, 0x02, 0x03, 0, 0, 5, 0, 0])
#
#komodo_send(base_id + 6, [0x01, 0x00, 0x00, 0, 0, 0, 0, 0])
#
#print(komodo_receive())
#
#
#komodo_close()
#
#
#
#
#





# -*- coding: utf-8 -*-
"""
Created on Wed Feb  5 13:33:33 2020

@author: butkus
"""
import json
import urllib
import urllib.error
import urllib.request
import urllib.parse
import time

class HTTP_methods:    
    def _get (self, command):
        time_init = time.perf_counter()
        try:
            data = json.loads(urllib.request.urlopen(self.url+command).read().decode('utf-8'))
        except urllib.error.HTTPError as e:
            error_string = e.read().decode('utf-8', 'ignore')
            print (error_string)
            return error_string
        
        if not self.silent:
            print (command + " : {:.3f} ms".format((time.perf_counter()-time_init)*1000))
        return data
        
    def _put (self, command, data):
        time_init = time.perf_counter()
        try:
            post_url = urllib.request.Request(url=self.url+command, data=data.encode('utf-8'), method='PUT')
    
            post_url.add_header('Content-Type', 'application/json')
            
            with urllib.request.urlopen(post_url) as f:
                pass
            if not self.silent:
                print (command + " : {:.3f} ms".format((time.perf_counter()-time_init)*1000))
            return f.status            
        except urllib.error.HTTPError as e:
            error_string = e.read().decode('utf-8', 'ignore')
            print (error_string)
            return error_string
        
        if not self.silent:
            print (command + " : {:.3f} ms".format((time.perf_counter()-time_init)*1000))
        return 200
    
        
    def _post (self, command, details={}):
        time_init = time.perf_counter()
        try:
            post_details = urllib.parse.urlencode(details).encode('UTF-8')
            
            post_url = urllib.request.Request(self.url+command, post_details)
            
            res = urllib.request.urlopen(post_url).read().decode('utf-8', 'ignore')
        except urllib.error.HTTPError as e:
            error_string = e.read().decode('utf-8', 'ignore')
            print (error_string)
            return error_string
        
        if not self.silent:
            print (command + " : {:.3f} ms".format((time.perf_counter()-time_init)*1000))
        return res


# -*- coding: utf-8 -*-
"""
Created on Wed Feb  5 13:33:33 2020

@author: butkus
"""
import json
import socket
import time

class UdpLocator:
    """
    Locates Topas4 devices on the same local area network using UDP multicast
    Identifier:
        ALL
        
    """    
    def locate(self, identifier = 'ALL'):
        # Create a UDP socket
        sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)

        multicastAddress = ('239.0.0.181', 7415)
        localHostAddress = ('127.255.255.255', 7415) #works as loopback broadcast for reused sockets
        message = (identifier + '?').encode('UTF-8')
        devices = []
        
        # Send data both to multicast address and to localhost
        # localhost is for cases when server and client applications are running on the same PC, and PC might be not connected to the network
        sock.sendto(message, multicastAddress)
        sock.sendto(message, localHostAddress)

        while (True):
            sock.settimeout(2.0)
            sock
            try:
                data, sender = sock.recvfrom(4096)
            except socket.timeout:
                break
            
            try:
                description = json.loads(data.decode('UTF8'))
                description['IpAddress'] = sender[0]
                
                if 'SenderGuid' in description.keys():
                    description['SenderGUID'] = description['SenderGuid']
                    del description['SenderGuid']
                                    
                if 'SenderGUID' in description.keys():
                    devices.append(description)
                    
            except json.decoder.JSONDecodeError:
                print('bad data received by locator')
           
        sock.close()
        seen = set()
        #multiple answers from the same device are possible, and certain if server is located on the same PC
        uniqDevices = [obj for obj in devices if obj['SenderGUID'] not in seen and not seen.add(obj['SenderGUID'])]
                                                   
        return uniqDevices
    
if __name__ == '__main__':
    loc = UdpLocator()
    devs = loc.locate('ALL')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#==========================================================================
# Harpia REST API Interface
#--------------------------------------------------------------------------
# Copyright (c) 2018 Light Conversion (UAB MGF "Šviesos konversija")
# All rights reserved.
# www.lightcon.com
#==========================================================================

from ..common import HTTP_methods

class BeamAlignmentSystem(HTTP_methods):    
    silent = True
    connected = False
            
    def __init__ (self, ip_address, port=20050, version='v1'):
        self.url = 'http://{}:{}/{}/'.format(ip_address, port, version)        
        self.connected = True;
        if self.connected:
            print ('Beam Alignment System initialized at', self.url)
        
    #==============================================================================
    # /Basic
    #==============================================================================    
    def get_radius(self, system, quadrant):
        return self._get(str(system) + '/Quadrant/'+str(quadrant)+'/Radius')                
    
    def get_quadrant(self, system, quadrant):
        return self._get(str(system) + '/Quadrant/'+str(quadrant))
    
    def align_whole_system(self, system):
        self._post(str(system) + '/AlignSystem')
        
    def get_aligning(self, system):
        return self._get(str(system) + '/Aligning')
    
    def set_autoalign_on(self, system):
        self._put(str(system)+'/AutoAlign', '1')
        
    def set_autoalign_off(self, system):
        self._put(str(system)+'/AutoAlign', '0')
        
    def get_actuator_position(self, system, actuator):
        return {'X': self._get(str(system) +'/Actuator/'+str(actuator)+'/Position/X'),
                'Y': self._get(str(system) + '/Actuator/'+str(actuator)+'/Position/Y')}
        
    def set_actuator_position(self, system, actuator, x, y):
        self._post(str(system) + '/Actuator/'+str(actuator)+'/Position/'+str(x)+'/'+str(y))    


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#==========================================================================
# Fast DAQ DLL Wrapper
#--------------------------------------------------------------------------
# Copyright (c) 2020 Light Conversion, UAB
# All rights reserved.
# www.lightcon.com
#==========================================================================

import clr   # pip install pythonnet
import numpy as np
import time

import os
dirname = os.path.dirname(__file__)
clr.AddReference(os.path.join(dirname, "LightConversion.Hardware.FastDaq.dll"))
clr.AddReference(os.path.join(dirname, "LightConversion.Abstractions.dll"))
clr.AddReference(os.path.join(dirname, "System.Threading.dll"))
clr.AddReference("mscorlib.dll")

from LightConversion.Hardware.FastDaq import FastDaq
from LightConversion.Abstractions import ClockChannel, SampleClockActiveEdge, TriggerChannel, DaqException
from System.Threading import CancellationTokenSource, CancellationToken
from System import TimeoutException

class FastDaqWrapper:
    '''
    DLL wrapper of FastDaq one-channel 2.5MHz ADC board.
    '''
    daq = None
    DaqException = DaqException
    TimeoutException = TimeoutException
    def __init__(self, device = 'Dev0'):
        devices = FastDaq.Devices
        if (device in devices):
            daq = FastDaq.LoadDevice(device)
            daq.Initialize()
            if daq.Connected:
                print ("Initialized", device, daq.Device.ProductType, daq.Device.SerialNumber)            
                self.daq = daq
                time.sleep(0.1)
            return
        
        print ("Harpia DAQ not found")

    def get_devices(self = None):
        return FastDaq.Devices

    def is_connected(self):
        return self.daq.Connected or False
        
    def set_timeout(self, timeout):
        '''
        Sets timeout in milliseconds
        
        Parameters
        ----------
        timeout : number
            Timeout value, given in milliseconds
        '''
        self.daq.AcquisitionTimeout = int(timeout)
        
    def configure_sample_clock(self, channel, active_edge):
        '''
        Configures clock for ADC acquisition
        
        Parameters
        ----------
        channel : str
            'internal' or 'PFI0'
        active_edge : str
            'rising' or 'falling'        
        '''
        my_clock_channel = ClockChannel.Internal
        if (channel == 'internal'):
            my_clock_channel = ClockChannel.Internal
        elif ((channel == 'PFI0')):
            my_clock_channel = ClockChannel.PFI0
            
        my_clock_active_edge = SampleClockActiveEdge.Rising
        if (active_edge == 'rising'):
            my_clock_active_edge = SampleClockActiveEdge.Rising
        else:
            my_clock_active_edge = SampleClockActiveEdge.Falling            
            
        self.daq.ConfigureSampleClock(my_clock_channel, 2.5e6, my_clock_active_edge, 1000)
            
            
    def configure_start_trigger(self, channel):
        '''
        Configures trigger for ADC acquisition
        
        Parameters
        ----------
        channel : str
            'internal' or 'PFI0'
        '''
        my_trigger_channel = TriggerChannel.Internal
        if (channel == 'internal'):
            my_trigger_channel = TriggerChannel.Internal
        elif ((channel == 'PFI0')):
            my_trigger_channel = TriggerChannel.PFI0

        self.daq.ConfigureStartTrigger(my_trigger_channel)
    
    
    def get_daq_data(self, n):
        '''
        Acquires n samples per enabled channel.
        
        Parameters
        ----------
        n : number
            Number of datapoints to acquire per channel
        
        Returns
        -------
        list (n):
            Values in volts
        '''
        self.daq.DatapointsPerChannel = n
        cts = CancellationTokenSource()
        ct = cts.Token

        data = self.daq.GetAnalogChannelsData(ct)        
        data = [item for item in data]        
        return data
    
    def set_external_trigger_delay(self, delay):
        '''
        Sets external trigger delay for sampling.
        
        Parameters
        ----------
        delay : number
            Delay value in nanoseconds
        '''
        self.daq.ExternalTriggerDelay = delay
        
    def close(self):
        self.daq.Dispose()


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#==========================================================================
# Harpia REST API Interface
#--------------------------------------------------------------------------
# Copyright (c) 2018 Light Conversion (UAB MGF "Šviesos konversija")
# All rights reserved.
# www.lightcon.com
#==========================================================================

import json
import socket
from ..common.http_methods import HTTP_methods
    
class Harpia (HTTP_methods):
    """REST API interaction logic with Harpia spectrometer"""
    
    silent = True
    connected = False
            
    def __init__ (self, ip_address, port=20050, version='v1'):
        self.url = 'http://{}:{}/{}/'.format(ip_address, port, version)        
        self.connected = self._get('Basic') != {}
        if self.connected:
            print ('Harpia initialized at', self.url)
        
    #==============================================================================
    # /Basic
    #==============================================================================
    
    def reload_factory_defaults(self):
        self._post('Basic/ReloadFactoryDefaults')

    def pump_probe_signal(self, close_shutters = True, target_delay = None):
        query = 'Basic/PumpProbeSpectrum/' + str(close_shutters).lower() 
        if target_delay != None:
            query = query + '/' + str(target_delay)            
        return self._get(query)
    
    def background_raw_signal(self):
        return self._get('Basic/BackgroundRawSignal')
    
    def raw_signal(self):
        return self._get('Basic/RawSignal')

    def spectra_per_acquisition(self):
        return self._get('Basic/NumberOfSpectraPerAcquisition')
    
    def set_spectra_per_acquisition(self, number):
        self._put('Basic/NumberOfSpectraPerAcquisition', str(number))
        
    def datapoints_per_spectrum(self):
        return self._get('Basic/DatapointsPerSpectrum')
    
    def wavelength_axis(self):
        return self._get('Basic/WavelengthAxis')    

    #==============================================================================
    # /Detectors
    #==============================================================================
    
    def available_detectors(self):
        return self._get('Detectors/AvailableDetectors')
    
    def current_detector(self):
        return self._get('Detectors/CurrentDetector')
    
    def set_current_detector(self, detector_number):
        self._put('Detectors/CurrentDetector', str(detector_number))

    #==============================================================================
    # /Shutters
    #==============================================================================
    def open_pump_shutter(self):
        self._post('Shutters/OpenPumpShutter')
        
    def close_pump_shutter(self):
        self._post('Shutters/ClosePumpShutter')
                    
    def open_probe_shutter(self):
        self._post('Shutters/OpenProbeShutter')
        
    def close_probe_shutter(self):
        self._post('Shutters/CloseProbeShutter')
        
    def open_gate_shutter(self):
        self._post('Shutters/OpenGateShutter')
        
    def close_gate_shutter(self):
        self._post('Shutters/OpenGateShutter')
        
    def open_third_beam_shutter(self):
        self._post('Shutters/OpenThirdBeamShutter')
        
    def close_pump_open_probe_shutters(self):
        self._post('Shutters/ClosePumpOpenProbeShutters')
        
    def open_pump_close_probe_shutters(self):
        self._post('Shutters/OpenPumpCloseProbeShutters')
        
    def open_all_shutters(self):
        self._post('Shutters/OpenAllShutters')
        
    def close_all_shutters(self):
        self._post('Shutters/CloseAllShutters')
        
        
    #==============================================================================
    # /Chopper
    #==============================================================================
        
    def chopper_is_on(self):
        return self._get('Chopper/IsOn')
    
    def chopper_start(self):
        self._post('Chopper/Start')
    
    def chopper_stop(self):
        self._post('Chopper/Stop')
    
    def chopper_is_locked(self):
        return self._get('Chopper/IsLocked')
    
    def chopper_clock_source(self):
        return self._get('Chopper/ClockSource')
    
    def set_chopper_clock_source_internal(self):
        self._put('Chopper/ClockSource', '\"internal\"')
        
    def set_chopper_clock_source_external(self):
        self._put('Chopper/ClockSource', '\"external\"')

    def chopper_clock_divider(self):
        return self._get('Chopper/ClockDivider')
    
    def set_chopper_clock_divider(self, divider):
        self._put('Chopper/ClockDivider', str(divider))
        
    def chopper_actual_frequency(self):
        return self._get('Chopper/ActualFrequency')
    
    def chopper_target_frequency(self):
        return self._get('Chopper/ActualFrequency')
    
    def set_chopper_target_frequency(self, frequency):
        self._put('Chopper/TargetFrequency', str(frequency))
        
    def chopper_phase_error(self):
        return self._get('Chopper/PhaseError')
        
    def chopper_target_phase(self):
        return self._get('Chopper/TargetPhase')
    
    def set_chopper_target_phase(self, phase):
        self._put('Chopper/TargetPhase', str(phase))
        
    #==============================================================================
    # /DelayLine 
    #==============================================================================
    
    def delay_line_actual_delay(self):
        return self._get('DelayLine/ActualDelay')
    
    def delay_line_target_delay(self):
        return self._get('DelayLine/TargetDelay')    
    
    def set_delay_line_target_delay(self, delay):
        self._put('DelayLine/TargetDelay', str(delay))
        
    def delay_line_actual_distance(self, distance):
        self._put('DelayLine/ActualDistance', str(distance))
        
    def set_delay_line_target_distance(self, distance):
        self._put('DelayLine/TargetDistance', str(distance))
        
    def delay_line_go_to_zero(self):
        self._post('DelayLine/GoToZero')
        
    def delay_line_set_as_zero(self):
        self._post('DelayLine/SetAsZero')

    #==============================================================================
    # /CrystalRotator
    #==============================================================================
    
    def crystal_rotator_is_moving(self):
        return self._get('CrystalRotator/IsMoving')
        
    def crystal_rotator_start(self):
        self._post('CrystalRotator/Start')

    def crystal_rotator_stop(self):
        self._post('CrystalRotator/Stop')

    #==============================================================================
    # /SampleMover
    #==============================================================================

    def sample_mover_go_to_zero(self):
        self._post('SampleMover/GoToZero')
        
    def sample_mover_go_to_xy_and_wait_for_stop(self, x, y):
        query = 'SampleMover/GoToXYAndWaitForStop/' + str(x) + '/' + str(y)            
        self._post(query)

    def set_sample_mover_velocity(self, velocity):
        self.put('SampleMover/Velocity', str(velocity))
        
    def sample_mover_velocity(self):
        return self._get('SampleMover/Velocity')
    
    
    #==============================================================================
    # /MotorizedSpectrograph    
    #==============================================================================
    
    def motorized_spectrograph_wavelength(self):
        return self._get('MotorizedSpectrograph/Wavelength')
        
    def set_motorized_spectrograph_wavelength(self, wavelength):
        self._put('MotorizedSpectrograph/Wavelength', str(wavelength))

    def motorized_spectrograph_output_port(self):
        return self._get('MotorizedSpectrograph/OutputPort')
        
    def set_motorized_spectrograph_output_port(self, port):
        self._put('MotorizedSpectrograph/OutputPort', port)
    
    def motorized_spectrograph_detector_offset(self):
        return self._get('MotorizedSpectrograph/DetectorOffset')
        
    def set_motorized_spectrograph_detector_offset(self, offset):
        self._put('MotorizedSpectrograph/DetectorOffset', str(offset))

    def motorized_spectrograph_grating_offset(self):
        return self._get('MotorizedSpectrograph/GratingOffset')
        
    def set_motorized_spectrograph_grating_offset(self, offset):
        self._put('MotorizedSpectrograph/GratingOffset', str(offset))
        
    def motorized_spectrograph_focus(self):
        return self._get('MotorizedSpectrograph/Focus')
        
    def set_motorized_spectrograph_focus(self, focus):
        self._put('MotorizedSpectrograph/Focus', str(focus))

    def motorized_spectrograph_gratings_info(self):
        return self._get('MotorizedSpectrograph/GratingsInfo')

    def motorized_spectrograph_grating(self):
        return self._get('MotorizedSpectrograph/Grating')
        
    def set_motorized_spectrograph_grating(self, grating):
        self._put('MotorizedSpectrograph/Grating', str(grating))
        
    #==============================================================================
    # /BerekRotator
    #==============================================================================
    
    def berek_rotator_go_to_zero(self):
        self._post('BerekRotator/GoToZero')
        
    def berek_rotator_actual_rotate_angle(self):
        return self._get('BerekRotator/ActualRotateAngle')
    
    def berek_rotator_actual_tilt_angle(self):
        return self._get('BerekRotator/ActualTiltAngle')
    
    def set_berek_rotator_target_rotate_angle(self, angle):
        self._put('BerekRotator/TargetRotateAngle', str(angle))
        
    def set_berek_rotator_target_tilt_angle(self, angle):
        self._put('BerekRotator/TargetTiltAngle', str(angle))        
    
    #==============================================================================
    # /PumpMotorizedVNDF
    #==============================================================================
    
    def pump_vndf_start_tune(self):
        self._post('PumpMotorizedVNDF/StartTune')
    
    def pump_vndf_stop_tune(self):
        self._post('PumpMotorizedVNDF/StopTune')
        
    def pump_vndf_go_by_percent(self, percent):
        self._post('PumpMotorizedVNDF/GoByPercent/' + str(percent))
        
    def pump_vndf_go_to_zero(self):
        self._post('PumpMotorizedVNDF/GoToZero')
        
    def pump_vndf_angle(self):
        return self._get('PumpMotorizedVNDF/Angle')
    
    def set_pump_vndf_angle(self, angle):
        self._put('PumpMotorizedVNDF/Angle', str(angle))

    def pump_vndf_transmittance(self):
        return self._get('PumpMotorizedVNDF/Transmittance')
    
    def set_pump_vndf_transmittance(self, transmittance):
        self._put('PumpMotorizedVNDF/Transmittance', str(transmittance))

    #==============================================================================
    # /ProbeMotorizedVNDF
    #==============================================================================
    
    def probe_vndf_start_tune(self):
        self._post('ProbeMotorizedVNDF/StartTune')
    
    def probe_vndf_stop_tune(self):
        self._post('ProbeMotorizedVNDF/StopTune')
        
    def probe_vndf_go_by_percent(self, percent):
        self._post('ProbeMotorizedVNDF/GoByPercent/' + str(percent))
        
    def probe_vndf_go_to_zero(self):
        self._post('ProbeMotorizedVNDF/GoToZero')
        
    def probe_vndf_angle(self):
        return self._get('ProbeMotorizedVNDF/Angle')
    
    def set_probe_vndf_angle(self, angle):
        self._put('ProbeMotorizedVNDF/Angle', str(angle))

    def probe_vndf_transmittance(self):
        return self._get('ProbeMotorizedVNDF/Transmittance')
    
    def set_probe_vndf_transmittance(self, transmittance):
        self._put('ProbeMotorizedVNDF/Transmittance', str(transmittance))

    #==============================================================================
    # /DetectorMotorizedVNDF
    #==============================================================================
    
    def detector_vndf_start_tune(self):
        self._post('DetectorMotorizedVNDF/StartTune')
    
    def detector_vndf_stop_tune(self):
        self._post('DetectorMotorizedVNDF/StopTune')
        
    def detector_vndf_go_by_percent(self, percent):
        self._post('DetectorMotorizedVNDF/GoByPercent/' + str(percent))
        
    def detector_vndf_go_to_zero(self):
        self._post('DetectorMotorizedVNDF/GoToZero')
        
    def detector_vndf_angle(self):
        return self._get('DetectorMotorizedVNDF/Angle')
    
    def set_detector_vndf_angle(self, angle):
        self._put('DetectorMotorizedVNDF/Angle', str(angle))

    def detector_vndf_transmittance(self):
        return self._get('DetectorMotorizedVNDF/Transmittance')
    
    def set_detector_vndf_transmittance(self, transmittance):
        self._put('DetectorMotorizedVNDF/Transmittance', str(transmittance))        


    #==============================================================================
    # /InGaAsDetector
    #==============================================================================
    
    def ingaas_detector_integration_time(self):
        return self._get('InGaAsDetector/IntegrationTime')
    
    def set_ingaas_detector_integration_time(self, time):
        self._put('InGaAsDetector/IntegrationTime', str(time))



    #==============================================================================
    # /HarpiaTF
    #==============================================================================

    def harpia_tf_spcm_trace(self):
        return self._get('/HarpiaTF/SPCM/Trace')

    def harpia_tf_spcm_rates(self):
        return self._get('/HarpiaTF/SPCM/Rates')
    
    def harpia_tf_spcm_time_axis(self):
        return self._get('/HarpiaTF/SPCM/TimeAxis')

    #==============================================================================
    # /TimingController
    #==============================================================================

    def set_timing_controller_frequency(self, frequency):
        return self._put('TimingController/Frequency', str(frequency))
    
    def timing_controller_frequency(self):
        return self._get('TimingController/Frequency')

    #==============================================================================
    # /ProbeSource
    #==============================================================================
    
    def probe_source_temperature(self):
        return self._get('ProbeSource/Temperature')
    
    def probe_source_turn_on(self):
        return self._post('ProbeSource/TurnOn')

    def probe_source_turn_off(self):
        return self._post('ProbeSource/TurnOff')
    
    def set_probe_source_internal_pulse_frequency(self, frequency):
        return self._put('ProbeSource/InternalPulseFrequency', str(frequency))
    
    def set_probe_source_trigger_mode_internal(self):
        self._put('ProbeSource/TriggerMode', r'"internal"')
        
    def set_probe_source_trigger_mode_external(self):
        self._put('ProbeSource/TriggerMode', r'"external"')
        
    
    
    #==============================================================================
    #     /FlashPhotolysis
    #==============================================================================
        
    def flash_photolysis_raw_signal(self):
        return self._get('FlashPhotolysis/RawSignal')
    
    def flash_photolysis_pump_probe_spectrum(self, close_shutters = True, target_delay = None):
        query = 'FlashPhotolysis/PumpProbeSpectrum/' + str(close_shutters).lower() 
        if target_delay != None:
            query = query + '/' + str(target_delay)            
        return self._get(query)
    
    
    
    #==============================================================================
    #     /HarpiaTB/DelayLine
    #==============================================================================

    def harpiatb_delay_line_actual_delay(self):
        return self._get('HarpiaTB/DelayLine/ActualDelay')
    
    def harpiatb_delay_line_target_delay(self):
        return self._get('HarpiaTB/DelayLine/TargetDelay')    
    
    def harpiatb_set_delay_line_target_delay(self, delay):
        self._put('HarpiaTB/DelayLine/TargetDelay', str(delay))
        
    def harpiatb_delay_line_go_to_zero(self):
        self._post('HarpiaTB/DelayLine/GoToZero')
        
    def harpiatb_delay_line_set_as_zero(self):
        self._post('HarpiaTB/DelayLine/SetAsZero')
        
        
    #==============================================================================
    #    /HarpiaTB/Chopper
    #==============================================================================
        
    def harpiatb_chopper_is_on(self):
        return self._get('HarpiaTB/Chopper/Chopper/IsOn')
    
    def harpiatb_chopper_start(self):
        self._post('HarpiaTB/Chopper/Chopper/Start')
    
    def harpiatb_chopper_stop(self):
        self._post('HarpiaTB/Chopper/Chopper/Stop')
    
    def harpiatb_chopper_is_locked(self):
        return self._get('HarpiaTB/Chopper/Chopper/IsLocked')
    
    def harpiatb_chopper_clock_source(self):
        return self._get('HarpiaTB/Chopper/Chopper/ClockSource')
    
    def harpiatb_set_chopper_clock_source_internal(self):
        self._put('HarpiaTB/Chopper/Chopper/ClockSource', '\"internal\"')
        
    def harpiatb_set_chopper_clock_source_external(self):
        self._put('HarpiaTB/Chopper/Chopper/ClockSource', '\"external\"')

    def harpiatb_chopper_clock_divider(self):
        return self._get('HarpiaTB/Chopper/Chopper/ClockDivider')
    
    def harpiatb_set_chopper_clock_divider(self, divider):
        self._put('HarpiaTB/Chopper/Chopper/ClockDivider', str(divider))
        
    def harpiatb_chopper_actual_frequency(self):
        return self._get('HarpiaTB/Chopper/Chopper/ActualFrequency')
    
    def harpiatb_chopper_target_frequency(self):
        return self._get('HarpiaTB/Chopper/Chopper/ActualFrequency')
    
    def harpiatb_set_chopper_target_frequency(self, frequency):
        self._put('HarpiaTB/Chopper/Chopper/TargetFrequency', str(frequency))
        
    def harpiatb_chopper_phase_error(self):
        return self._get('HarpiaTB/Chopper/Chopper/PhaseError')
        
    def harpiatb_chopper_target_phase(self):
        return self._get('HarpiaTB/Chopper/Chopper/TargetPhase')
    
    def harpiatb_set_chopper_target_phase(self, phase):
        self._put('HarpiaTB/Chopper/Chopper/TargetPhase', str(phase))
        
    #==============================================================================
    #      /BeamAlignmentSystem
    #==============================================================================
        
    def internal_quadrant_detectors(self):
        return self._get('InternalQuadrantDetectors')
    
    def internal_quadrant_detectors_trigger_delay(self):
        return self._get('InternalQuadrantDetectors/ExternalTriggerDelay')
    
    def set_internal_quadrant_detectors_trigger_delay(self, delay):
        return self._put('InternalQuadrantDetectors/ExternalTriggerDelay', str(delay))
    
    #==============================================================================
    #     /Microscope
    #==============================================================================
    
    def microscope_camera_alldata(self, index = None):
        return self._get('Microscope/Camera/AllData')
    
    def microscope_sample_stage_position_X(self):
        return self._get('Microscope/SampleStagePositionX')
    
    def microscope_set_sample_stage_position_X(self, position):
        return self._put('Microscope/SampleStagePositionX', str(position))

    def microscope_sample_stage_position_Y(self):
        return self._get('Microscope/SampleStagePositionY')
    
    def microscope_set_sample_stage_position_Y(self, position):
        return self._put('Microscope/SampleStagePositionY', str(position))    
    
    def microscope_sample_stage_position_Z(self):
        return self._get('Microscope/SampleStagePositionZ')
    
    def microscope_set_sample_stage_position_Z(self, position):
        return self._put('Microscope/SampleStagePositionZ', str(position))    

    #==============================================================================
    #     /MotorizedPumpMirror
    #==============================================================================
    
    def motorized_pump_mirror(self):
        return self._get('MotorizedPumpMirror')
    
    def motorized_pump_mirror_position_X(self):
        return self._get('MotorizedPumpMirror/PositionX')
    
    def motorized_pump_mirror_set_position_X(self, position):
        return self._put('MotorizedPumpMirror/PositionX', str(position))

    def motorized_pump_mirror_set_relative_position_X(self, position):
        return self._put('MotorizedPumpMirror/RelativePositionX', str(position))
    
    def motorized_pump_mirror_position_Y(self):
        return self._get('MotorizedPumpMirror/PositionY')
    
    def motorized_pump_mirror_set_position_Y(self, position):
        return self._put('MotorizedPumpMirror/PositionY', str(position))

    def motorized_pump_mirror_set_relative_position_Y(self, position):
        return self._put('MotorizedPumpMirror/RelativePositionY', str(position))
    
    
    # =============================================================================
    # /ZScan    
    # =============================================================================
    
    def zscan_response (self):
        return self._get('ZScan/Response')
    
    def zscan_set_samples_per_point (self, samples):
        self._put('ZScan/SamplesPerPoint', str(samples))
    
    def zscan_set_position (self, position):
        self._put('ZScan/Position', str(position))
    
    def zscan_position (self):
        return self._get('ZScan/Position')

    


# -*- coding: utf-8 -*-
'''
Created on Wed Feb  5 16:19:16 2020

@author: butkus
'''
import setuptools

with open('README.md', 'r') as fh:
    long_description = fh.read()

with open('../latest_version.txt', 'r') as f:
    content = f.read()
    version_major, version_minor, version_revision = content.split('.')        

version_revision = int(version_revision) + 1;
new_version_string = '{:}.{:}.{:}'.format(version_major, version_minor, version_revision)

with open('../latest_version.txt', 'w') as f:
    f.write(new_version_string)
    
setuptools.setup(
    name='lightcon', # Replace with your own username
    version=new_version_string,
    author='Vytautas Butkus',
    author_email='vytautas.butkus@lightcon.com',
    description='A set of APIs to Light Conversion devices',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://bitbucket.org/harpiasoftware/light-conversion-apis.git',
    install_requires = ['pythonnet>=2.5.0tcon', 'pyserial', 'numpy'],
    packages=setuptools.find_packages(),
    data_files = [('Lib/site-packages/lightcon/harpia_daq', 
                   ['lightcon/harpia_daq/LightConversion.Hardware.HarpiaDaq.dll', 
                    'lightcon/harpia_daq/FTD2XX_NET.dll', 
                    'lightcon/harpia_daq/LightConversion.Abstractions.dll',
                    'lightcon/harpia_daq/System.Threading.dll']),
                ('Lib/site-packages/lightcon/fast_daq', 
                   ['lightcon/fast_daq/LightConversion.Hardware.FastDaq.dll', 
                    'lightcon/fast_daq/FTD2XX_NET.dll', 
                    'lightcon/fast_daq/LightConversion.Abstractions.dll',
                    'lightcon/fast_daq/System.Threading.dll']),
                ('Lib/site-packages/lightcon/common', 
                   ['lightcon/common/komodo.dll'])],
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Development Status :: 1 - Planning',
    ],
    project_urls = {
            'Examples': 'https://bitbucket.org/harpiasoftware/light-conversion-apis/src/master/examples/'},
    python_requires='>=3.6',
)




# -*- coding: utf-8 -*-
"""
Created on Mon Feb 10 08:50:15 2020

@author: butkus
"""
import serial
import time
import struct
from lightcon.common.serial_tools import get_serial_ports

class TimingControllerV2:
    serial_port = None
    device_type = 'N/A'
    hardware_id = 'N/A'
    firmware_version = 'N/A'
    connected = False
    baud_rate = 1000000
    ser = None
        
    def __init__ (self):
        return
    
    def __dispose__ (self):
        if self.ser:
            self.ser.close()
    
    def _is_device_supported(self, serial_port):
        try:
            self.serial_port = serial_port
            response = self._read_(b'?\n')
            self.ser.close()
            self.ser = None
            response = response.decode()[:-1].replace('\r', '').replace('\n','')
        except:
            response = ''
        return [response != '', response]      
        
    
    def connect(self, serial_port = None, index = None):
        if serial_port is None:
            ports = get_serial_ports()
            supported_devices = [[port] + self._is_device_supported(port) for port in ports]
            supported_devices  = [tuple(dev) for dev in supported_devices if dev[1]]
            if len(supported_devices) == 1:
                self.serial_port = supported_devices[0][0]
            elif len(supported_devices) > 1:
                if index is None:
                    print ('Multiple supported devices found: \n' + '\n'.join('\t'.join([str(i), entry[0], entry[2]]) for i, entry in enumerate(supported_devices)))
                    print ('Invoke function with the correspondindg index value')
                else:
                    self.serial_port = supported_devices[index][0]
            else:
                print ('No supported devices found on serial bus')
                return
        else:
            self.serial_port = serial_port
        
        try:
            response = self._read_(b'?\n')
            response = response.decode()[:-1].replace('\r', '').replace('\n','')
        except:
            response = ''
        if response != '':
            elements = response.split(':')
            if len(elements) == 3:
                self.device_type = str(elements[0])
                self.hardware_id = elements[1]
                self.firmware_version = elements[2]
                
                self.connected = True
                print ('Connected to {:} on {:} ({:}, firmware version: {:})'.format(self.device_type, self.serial_port, self.hardware_id, self.firmware_version))
                return
        
        print ('Could not connect to timing controller on {:}'.format(self.serial_port))
                        
    def _read_(self, message):
        '''For low-level serial messaging'''
        try:
            if self.ser == None:
                self.ser = serial.Serial(self.serial_port, baudrate=1000000)
            self.ser.reset_input_buffer()
            self.ser.reset_output_buffer()      
        except serial.SerialException:
            return '\n'.encode()
        
        self.ser.write(message)
        
        cnt = 0
        while self.ser.in_waiting == 0 and cnt < 20:
            cnt = cnt + 1
            time.sleep(0.01)
            
        response = self.ser.read(self.ser.in_waiting)
        
        
        
        return response
        
    def _write_(self, message):
        '''For low-level serial messaging'''
        try:
            if self.ser == None:
                self.ser = serial.Serial(self.serial_port, baudrate=1000000)
            self.ser.reset_input_buffer()
            self.ser.reset_output_buffer()      
        except serial.SerialException:
            return '\n'.encode()
        
        self.ser.write(message)        
    
    def set_delay(self, channel, delay):
        '''
        Sets delay of given channel
        
        Parameters
        ----------
        channel : number
            Channel 0-3
        delay : nubmer
            Delay value in nanoseconds
        '''
        return self._read_(bytes('set:del {:} {:}\n'.format(channel,delay), 'utf-8'))
        

    def get_delay(self, channel):
        return int(self._read_(bytes('get:del {:}\n'.format(channel), 'utf-8'))[:-1].decode())
    
    def get_output(self):
        return int(self._read_(bytes('get:output\n', 'utf-8'))[:-1].decode())
    
    def set_output(self, output):
        self._write_(bytes('set:output {:}\n'.format(output), 'utf-8'))
    
    def get_enabled(self, channel):
        output_status = self.get_output()
        flag = 1 << channel
        return output_status & flag != 0
    
    def enable_channel(self, channel):
        output_status = self.get_output()
        output = output_status | (1 << channel)
        self.set_output(output)
        
    def disable_channel(self, channel):
        output_status = self.get_output()
        output = output_status - (output_status & (1 << channel))
        self.set_output(output)
        
    def set_master_delay(self, delay):
        self._write_(bytes('set:masterdel {:}\n'.format(delay), 'utf-8'))
        
    def get_master_delay(self):
        return int(self._read_(bytes('get:masterdel\n', 'utf-8'))[:-1].decode())

    def get_master_frequency(self):
        return int(self._read_(bytes('get:extfreq\n', 'utf-8'))[:-1].decode())
            
    def read_stopwatch(self, n, flag = 62):
        '''
        Reads stopwatch values
        
        Parameters
        ----------
        n : number
            Number of start-stop events to capture
        flag : number
            Channel enabling flag
                bit0: no effect, trigger output cannot be turned off
                bit1: delay 1a control
                bit2: delay 1b control
                bit3: delay 2 control
                bit4: delay 3 control
                bit5: stopwatch arming control
        Returns
        -------
        delays : array(number)
            Array of timing values in microseconds
        '''
        if self.ser == None:
            self.ser = serial.Serial(self.serial_port, baudrate=1000000)
        self.ser.write(b'tdc:out 1\n')
        self.ser.write(bytes('set:output {:}\n'.format(flag), 'utf-8'))
        
        to_read = n * 4
        cnt = 0
        while self.ser.in_waiting < to_read:
            cnt = cnt+1
            if cnt > 100:
                print("TIMEOUT", self.ser.in_waiting, self.ser.read(self.ser.in_waiting))
                return
            time.sleep(0.01)
    
        self.ser.write(b'set:output 8\n')
        self.ser.write(b'tdc:out 0\n')        
        
        time.sleep(0.03)
        
        buffer = self.ser.read(to_read)
        
#        self.ser.close()
        
        delays = [struct.unpack('f', buffer[4*i:(4*i+4)])[0] for i in range(n)]
        print(buffer.hex())
        return delays
#    
if __name__ == '__main__':        
    tc = TimingControllerV2()
    tc.connect()
    out = tc.read_stopwatch(10)
    print(out)
    tc.__dispose__()



# -*- coding: utf-8 -*-
"""
Created on Mon Feb 10 09:16:00 2020

@author: butkus
"""
import serial
import sys
import glob

def get_serial_ports():
    """ Lists serial port names

        :raises EnvironmentError:
            On unsupported or unknown platforms
        :returns:
            A list of the serial ports available on the system
    """
    if sys.platform.startswith('win'):
        ports = ['COM%s' % (i + 1) for i in range(256)]
    elif sys.platform.startswith('linux') or sys.platform.startswith('cygwin'):
        # this excludes your current terminal "/dev/tty"
        ports = glob.glob('/dev/tty[A-Za-z]*')
    elif sys.platform.startswith('darwin'):
        ports = glob.glob('/dev/tty.*')
    else:
        raise EnvironmentError('Unsupported platform')

    result = []
    for port in ports:
        try:
            s = serial.Serial(port)
            s.close()
            result.append(port)
        except (OSError, serial.SerialException):
            pass
    return result


# -*- coding: utf-8 -*-
"""
Created on Mon Feb 10 08:50:15 2020

@author: butkus
"""
import serial
import time
import struct
from lightcon.common.serial_tools import get_serial_ports

class TimingController:
    serial_port = None
    device_type = 'N/A'
    hardware_id = 'N/A'
    firmware_version = 'N/A'
    connected = False
    baud_rate = 1000000
        
    def __init__ (self):
        return
    
    def _is_device_supported(self, serial_port):
        try:
            self.serial_port = serial_port
            response = self._read_(b'?\r\n')
            print(response)
            response = response.decode()[:-2]
        except:
            response = ''
        return [response != '', response]      
        
    
    def connect(self, serial_port = None, index = None):
        if serial_port is None:
            ports = get_serial_ports()
            supported_devices = [[port] + self._is_device_supported(port) for port in ports]
            supported_devices  = [tuple(dev) for dev in supported_devices if dev[1]]
            if len(supported_devices) == 1:
                self.serial_port = supported_devices[0][0]
            elif len(supported_devices) > 1:
                if index is None:
                    print ('Multiple supported devices found: \n' + '\n'.join('\t'.join([str(i), entry[0], entry[2]]) for i, entry in enumerate(supported_devices)))
                    print ('Invoke function with the correspondindg index value')
                else:
                    self.serial_port = supported_devices[index][0]
            else:
                print ('No supported devices found on serial bus')
                return
        else:
            self.serial_port = serial_port
        
        try:
            response = self._read_(b'?\r\n')
            print(response)
            response = response.decode()[:-2]
        except:
            response = ''
        if response != '':
            elements = response.split(':')
            if len(elements) == 3:
                self.device_type = str(elements[0])
                self.hardware_id = elements[1]
                self.firmware_version = elements[2]
                
                self.connected = True
                print ('Connected to {:} on {:} ({:}, firmware version: {:})'.format(self.device_type, self.serial_port, self.hardware_id, self.firmware_version))
                return
        
        print ('Could not connect to timing controller on {:}'.format(self.serial_port))
                        
    def _read_(self, message):
        '''For low-level serial messaging'''
        try:
            ser = serial.Serial(self.serial_port, baudrate=1000000)
        except serial.SerialException:
            return '\r\n'.encode()
        
        ser.write(message)
        
        time.sleep(0.1)
        response = ser.read(ser.in_waiting)
        
        ser.close()
        
        return response
        
    def set_delay(self, channel, delay):
        '''
        Sets delay of given channel
        
        Parameters
        ----------
        channel : number
            Channel 0-3
        delay : nubmer
            Delay value in nanoseconds
        '''
        return self._read_(bytes('set:del {:} {:}\r\n'.format(channel,delay), 'utf-8'))
    
    def set_frequency(self, frequency):
        '''
        Sets internal frequency
        
        Parameters
        ----------
        frequency : number
            Frequency in Hz        
        '''
        return self._read_(bytes('set:freq {:}\r\n'.format(frequency), 'utf-8'))

    def get_delay(self, channel):
        return struct.unpack('i', self._read_(bytes('read:del {:}\r\n'.format(channel), 'utf-8'))[:-2])[0]
    
    def get_enabled(self, channel):
        return struct.unpack('i', self._read_(bytes('read:en {:}\r\n'.format(channel), 'utf-8'))[:-2])[0]
    
    def get_frequency(self):
        return struct.unpack('i', self._read_(bytes('read:freq\r\n', 'utf-8'))[:-2])[0]
    
    def get_trigger_source(self):
        return struct.unpack('i', self._read_(bytes('read:trig\r\n', 'utf-8'))[:-2])[0]
    
    def set_trigger_external(self):
        return self._read_(bytes('set:trig 1\r\n', 'utf-8'))
    
    def set_trigger_internal(self):
        return self._read_(bytes('set:trig 0\r\n', 'utf-8'))
    
    def read_stopwatch(self, n, flag = 62):
        '''
        Reads stopwatch values
        
        Parameters
        ----------
        n : number
            Number of start-stop events to capture
        flag : number
            Channel enabling flag
                bit0: no effect, trigger output cannot be turned off
                bit1: delay 1a control
                bit2: delay 1b control
                bit3: delay 2 control
                bit4: delay 3 control
                bit5: stopwatch arming control
        Returns
        -------
        delays : array(number)
            Array of timing values in microseconds
        '''
        ser = serial.Serial(self.serial_port)
        ser.write(bytes('set:all {:}\r\n'.format(flag), 'utf-8'))
        
        to_read = n * 4
        while ser.in_waiting < to_read:
            time.sleep(0.01)
    
        ser.write(b'set:out 0\r\n')        
        buffer = ser.read(to_read)
        
        time.sleep(0.1)
        ser.reset_input_buffer()
        ser.reset_output_buffer()        
        ser.close()
        
        delays = [struct.unpack('f', buffer[4*i:(4*i+4)])[0] for i in range(n)]
        
        return delays
    
if __name__ == '__main__':        
    tc = TimingController()
    tc.connect()
    
#    if tc.connected:
#        tc.set_trigger_internal()
#        print ('Trigger source', 'EXTERNAL' if tc.get_trigger_source() == 1 else 'INTERNAL {:} Hz'.format(tc.get_frequency()))
#        print ('\n'.join(['Channel {:}, delay {:} ns, {:}'.format(i, tc.get_delay(i), 'ENABLED' if tc.get_enabled(i)==1 else 'DISABLED') for i in [1,2,3,4]]))
#        
#        tc._read_(b'set:pulse 1 10000\r\n')
        


# -*- coding: utf-8 -*-
"""
Created on Thu Jan 17 16:09:38 2019

@author: butkus

TO MAKE EVERYTHING WORK:
    1. Configure ethernet adapter to use static IP address 10.1.1.x, where x is anything from 1 to 255
    2. Run commands from http://nova/display/HAR/EthMotorBoard    
"""

import socket
import time

class EthMotorBoard:
    BUFFER_SIZE = 1024
    sock = None
    connected = False
    name = None
    timeout = 100
    fv = None
    ip_address = None
    max_position = 2**21-1
    status_registers = [(0x01,0x01,'HiZ'), (0x02,0x0,'BUSY'), (0x04,0x04, 'SW_F'), (0x08,0x08,'SW_ENV'), #(0x10, 0x10,'DIR'), 
                        (0x60,0x00,'Stopped'), (0x60,0x20,'Acceleration'), (0x60,0x40,'Deceleration'), (0x60,0x60,'Constant speed'), (0x80,0x80,'NOTPERF_CMD'),
                        (0x100,0x100,'WRONG_CMD'), (0x200,0x0,'OVLO'), (0x400,0x0,'TH_WRN'),(0x800,0x0,'TH_SD'),
                        (0x1000,0x0,'OCD'), (0x2000,0x0,'STEP_LOSS_A'), (0x4000,0x0,'STEP_LOSS_B'), (0x8000,0x8000,'SCK_MOD')]    
    ls_registers = [(0x01,0x01,'Left LS reached'), (0x02,0x02,'Right LS reached')]
    def __init__ (self, ip_address='10.1.1.0'):
        self.ip_address = ip_address           
                    
        self.name = self.send('GET BOARD_NAME')
        self.fv = self.send('FIRMWARE_VERSION')
        
        self.connected = self.fv != None
        
        if self.connected:
            print ('Successfullly connected to {:} (firmware version: {:})'.format(self.name, self.fv))
        else:
            print ('Motor board not found at {:}'.format(self.ip_address))
        
    def send(self, message, args=None):
        try:
            self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            self.sock.settimeout(self.timeout/1000)
            self.sock.connect((self.ip_address, 80))            
            if args is None:
                self.sock.send((str(message)+'\r\n').encode('UTF-8'))
            else:
                self.sock.send((str(message)+' '+' '.join([str(arg) for arg in args])+'\r\n').encode('UTF-8'))
            data = self.sock.recv(self.BUFFER_SIZE)
            self.sock.close()        
            return data[:-2].decode()
        except socket.timeout:
            return None
    def get_status(self, motor_index):
        status = int(self.send('GET STATUS', [motor_index]))
        ls_status = eval(self.send('GET LIMIT_SWITCH', [motor_index]))['Logical']
        return [stat for mask, val, stat in self.status_registers if status & mask == val] + [stat for mask, val, stat in self.ls_registers if ls_status & mask == val]   
                
    def wait_until_stopped(self, motor_index):
        repeat = True
        while repeat:
            repeat = int(self.send('GET STATUS ' + str(motor_index))) & 0x60 != 0
            time.sleep(0.05)    
